import warnings
from copy import copy

import numpy as np
from scipy import sparse

from . import smooth_atom, affine_smooth
from ..affine import (astransform, 
                      linear_transform, 
                      affine_transform)
from ..identity_quadratic import identity_quadratic
from ..atoms.seminorms import l1norm
from .cox import cox_loglike

from .glm import (glm,
                  gaussian_loglike,
                  logistic_loglike,
                  poisson_loglike,
                  cox_loglike,
                  huber_loss,
                  huber_svm)

class mglm(glm):

    """
    A multiresponse general linear model, usually a log-likelihood
    for response $Y$ whose mean is modelled through
    $X\beta$. 

    Usual examples are multivariate Gaussian (least squares regression),
    and multinomial regression.

    """

    def __init__(self, 
                 X, 
                 loss, 
                 quadratic=None, 
                 initial=None,
                 offset=None,
                 case_weights=None):

        """

        Parameters
        ----------

        X : ndarray((n,p))
            The design matrix.

        loss : `regreg.smooth.smooth_atom`
            The loss function that takes arguments the same
            size as `Y`. So, for Gaussian regression 
            the loss is just the map $\mu \mapsto \|\mu - Y\|^2_F/2$
            the Frobenius loss. Should have shape (p,q).

        quadratic : `regreg.identity_quadratic.identity_quadratic`
            Optional quadratic part added to objective.

        initial : ndarray
            An initial guess at the minimizer.

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        case_weights : ndarray
            Non-negative case weights

        """

        self.saturated_loss = loss
        self.data = X, loss.data
        self.affine_atom = affine_smooth(loss, X)
        if case_weights is None:
            case_weights = np.ones(X.shape[0])
        self.case_weights = case_weights
        smooth_atom.__init__(self,
                             (X.shape[1], loss.shape[1]),
                             coef=1.,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial)

    def subsample(self, idx):
        """
        Create a loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `mglm`
            Loss after discarding all
            cases not in `idx.

        """

        subsample_loss = copy(self)
        n = subsample_loss.saturated_loss.shape[0]

        idx_bool = np.zeros(n, np.bool)
        idx_bool[idx] = 1

        subsample_loss.case_weights *= idx_bool

        return subsample_loss
        
      
    def hessian(self, beta):
        """

        Compute the Hessian of the loss $ \nabla \ell(X\beta)$.
        NOT IMPLEMENTED.

        Parameters
        ----------

        beta : ndarray
            Parameters.
        """

        raise NotImplementedError

    def latexify(self, var=None, idx=''):
        return self.affine_atom.latexify(var=var, idx=idx)

    def __copy__(self):
        klass = self.__class__
        X, _ = self.data
        return klass(copy(X), 
                     copy(self.saturated_loss), 
                     quadratic=copy(self.quadratic),
                     initial=copy(self.coefs),
                     offset=copy(self.offset),
                     case_weights=copy(self.case_weights))
       
    @classmethod
    def multinomial(klass, 
                    X, 
                    successes, 
                    case_weights=None,
                    coef=1., 
                    saturated_offset=None,
                    offset=None,
                    quadratic=None, 
                    initial=None,
                    baseline=False):
        """
        Create a loss for a multinomial regression model.

        Parameters
        ----------

        X : [ndarray, `regreg.affine.affine_transform`]
            Design matrix

        successes : ndarray
            Responses (should be non-negative integers).

        case_weights : ndarray
            Non-negative case weights

        saturated_offset : ndarray (optional)
            Offset to be applied in saturated parameter space before 
            evaluating loss.

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        quadratic : `regreg.identity_quadratic.identity_quadratic` (optional)
            Optional quadratic to be added to objective.

        initial : ndarray
            Initial guess at coefficients.
           
        Returns
        -------

        mglm_obj : `regreg.mglm.mglm`
            General linear model loss.

        """

        if not baseline:
            loss = multinomial_loglike(successes.shape,
                                       successes,
                                       coef=coef,
                                       offset=saturated_offset)
        else:
            loss = multinomial_baseline_loglike((successes.shape[0],
                                                 successes.shape[1]-1),
                                                successes,
                                                coef=coef,
                                                offset=saturated_offset)

        return klass(X, 
                     loss,
                     offset=offset,
                     quadratic=quadratic,
                     initial=initial,
                     case_weights=case_weights)

    @classmethod
    def stacked(klass, 
                X, 
                losses,
                case_weights=None,
                coef=1., 
                offset=None,
                saturated_offset=None,
                quadratic=None, 
                initial=None):
        """
        Create a stacked loss. 

        Parameters
        ----------

        X : [ndarray, `regreg.affine.affine_transform`]
            Design matrix

        losses : sequence
            Sequence of saturated losses.

        coef : float
            Scaling to be put in front of loss.

        case_weights : ndarray
            Non-negative case weights

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        quadratic : `regreg.identity_quadratic.identity_quadratic` (optional)
            Optional quadratic to be added to objective.

        initial : ndarray
            Initial guess at coefficients.
           
        Returns
        -------

        mglm_obj : `regreg.mglm.mglm`
            General linear model loss.

        """

        loss = stacked_common_loglike(losses,
                                      successes,
                                      coef=coef,
                                      offset=saturated_offset)

        return klass(X, 
                     loss,
                     offset=offset,
                     quadratic=quadratic,
                     initial=initial,
                     case_weights=case_weights)

class stacked_common_loglike(smooth_atom):

    """
    A class for stacking `K=len(losses)` saturated losses with common
    shapes (roughly speaking) summed over losses.

    Roughly speaking a model of `K` independent measurements per individual.
    """

    objective_template = r"""\ell^{\text{logit}}\left(%(var)s\right)"""

    def __init__(self, 
                 losses,
                 coef=1., 
                 offset=None,
                 quadratic=None,
                 initial=None,
                 case_weights=None):

        shape = losses[0].shape + (len(losses),)
        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)

        assert(np.all([l.shape == losses[0].shape for l in losses]))
        self.data = np.array([l.data for l in losses]).T

        self._losses = losses
        self._gradient = np.zeros(self.shape)

        if case_weights is not None:
            if not np.all(case_weights >= 0):
                raise ValueError('case_weights should be non-negative')
            self.case_weights = np.asarray(case_weights)
            if self.case_weights.shape != self.shape[:1]:
                raise ValueError('case_weights should have same shape as response')
        else:
            self.case_weights = None

    def smooth_objective(self, 
                         natural_param, 
                         mode='both', 
                         check_feasibility=False,
                         case_weights=None):
        """

        Evaluate the smooth objective, computing its value, gradient or both.

        Parameters
        ----------

        natural_param : ndarray
            The current parameter values.

        mode : str
            One of ['func', 'grad', 'both']. 

        check_feasibility : bool
            If True, return `np.inf` when
            point is not feasible, i.e. when `natural_param` is not
            in the domain.

        Returns
        -------

        If `mode` is 'func' returns just the objective value 
        at `natural_param`, else if `mode` is 'grad' returns the gradient
        else returns both.
        """
        
        if case_weights is None:
            case_weights = np.ones(natural_param.shape[:1])
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        x = natural_param # shorthand

        x = self.apply_offset(x)
        if mode == 'grad':
            for i in range(len(self._losses)):
                self._gradient[:,i] = self._losses[i].smooth_objective(x[:,i], 
                                                                       'grad', 
                                                                       case_weights=cw) 
            return self.scale(self._gradient)
        elif mode == 'func':
            value = 0
            for i in range(len(self._losses)):
                value += self._losses[i].smooth_objective(x[:,i], 
                                                          'func', 
                                                          case_weights=cw)
            return self.scale(value)
        elif mode == 'both':
            value = 0
            for i in range(len(self._losses)):
                f, g = self._losses[i].smooth_objective(x[:,i], 
                                                        'both', 
                                                        case_weights=cw)
                self._gradient[:,i] = g
                value += f
            return self.scale(value), self.scale(self._gradient)
        else:
            raise ValueError("mode incorrectly specified")

    def get_data(self):
        return self._data

    def set_data(self, data):
        self._data = data

    data = property(get_data, set_data)

    def __copy__(self):
        return stacked_common_loglike(copy(self._losses),
                                      coef=self.coef,
                                      offset=copy(self.offset),
                                      quadratic=copy(self.quadratic),
                                      initial=copy(self.coefs),
                                      case_weights=copy(self.case_weights))

    def subsample(self, case_idx):
        """
        Create a saturated loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `smooth_atom`
            Loss after discarding all
            cases not in `idx.

        """
        loss_cp = copy(self)
        if loss_cp.case_weights is None:
            case_weights = loss_cp.case_weights = np.ones(self.shape[0])
        else:
            case_weights = loss_cp.case_weights

        idx_bool = np.zeros_like(case_weights, np.bool)
        idx_bool[case_idx] = 1

        case_weights *= idx_bool
        return loss_cp

    @classmethod
    def gaussian(klass,
                 responses,
                 case_weights=None,
                 coef=1., 
                 offset=None,
                 quadratic=None, 
                 initial=None):
        """
        Create a loss for a Gaussian regression model.

        Parameters
        ----------

        responses : ndarray
            Response vectors.

        case_weights : ndarray
            Non-negative case weights

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        quadratic : `regreg.identity_quadratic.identity_quadratic` (optional)
            Optional quadratic to be added to objective.

        initial : ndarray
            Initial guess at coefficients.
           
        Returns
        -------

        mglm_obj : `regreg.mglm.mglm`
            General linear model loss.

        """

        losses = [gaussian_loglike(response.shape,
                                   response,
                                   coef=coef)
                  for response in responses]

        return klass(losses,
                     offset=offset,
                     quadratic=quadratic,
                     initial=initial,
                     case_weights=case_weights)

    @classmethod
    def logistic(klass, 
                 successes, 
                 trials=None,
                 case_weights=None,
                 coef=1., 
                 offset=None,
                 quadratic=None, 
                 initial=None):
        """
        Create a loss for a logistic regression model.

        Parameters
        ----------

        successes : ndarray
            Responses (should be non-negative integers).

        trials : ndarray (optional)
            Number of trials for each success. If `None`,
            defaults to `np.ones_like(successes)`.

        case_weights : ndarray
            Non-negative case weights

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        quadratic : `regreg.identity_quadratic.identity_quadratic` (optional)
            Optional quadratic to be added to objective.

        initial : ndarray
            Initial guess at coefficients.
           
        Returns
        -------

        mglm_obj : `regreg.mglm.mglm`
            General linear model loss.

        """

        losses = [logistic_loglike(successes[i],
                                   trials[i],
                                   coef=coef)
                  for i in range(len(successes))]

        return klass(losses,
                     offset=offset,
                     quadratic=quadratic,
                     initial=initial,
                     case_weights=case_weights)

    @classmethod
    def poisson(klass,
                counts,
                case_weights=None,
                coef=1., 
                offset=None,
                quadratic=None, 
                initial=None):
        """
        Create a loss for a Poisson regression model.

        Parameters
        ----------

        counts : ndarray
            Response vector. Should be non-negative integers.

        case_weights : ndarray
            Non-negative case weights

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        quadratic : `regreg.identity_quadratic.identity_quadratic` (optional)
            Optional quadratic to be added to objective.

        initial : ndarray
            Initial guess at coefficients.
           
        Returns
        -------

        mglm_obj : `regreg.mglm.mglm`
            General linear model loss.

        """

        losses = [logistic_loglike(successes[i],
                                   trials[i],
                                   coef=coef)
                  for i in range(len(successes))]

        return klass(losses,
                     offset=offset,
                     quadratic=quadratic,
                     initial=initial,
                     case_weights=case_weights)

    @classmethod
    def huber(klass,
              X, 
              response,
              smoothing_parameter,
              case_weights=None,
              coef=1., 
              offset=None,
              quadratic=None, 
              initial=None):
        """
        Create a loss for a regression model using
        Huber loss.

        Parameters
        ----------

        response : ndarray
            Response vector. 

        smoothing_parameter : float
            Smoothing parameter for Huber loss.

        case_weights : ndarray
            Non-negative case weights

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        quadratic : `regreg.identity_quadratic.identity_quadratic` (optional)
            Optional quadratic to be added to objective.

        initial : ndarray
            Initial guess at coefficients.
           
        Returns
        -------

        mglm_obj : `regreg.mglm.mglm`
            General linear model loss.

        """

        losses = [huber_loss(response.shape,
                             response,
                             smoothing_parameter) 
                  for response in responses]

        return klass(losses,
                     offset=offset,
                     quadratic=quadratic,
                     initial=initial,
                     case_weights=case_weights)

    @classmethod
    def huber_svm(klass,
                  X, 
                  successes,
                  smoothing_parameter,
                  case_weights=None,
                  coef=1., 
                  offset=None,
                  quadratic=None, 
                  initial=None):
        """
        Create a loss for a regression model using
        Huber loss.

        Parameters
        ----------

        successes : ndarray
            Response vectors. 

        smoothing_parameter : float
            Smoothing parameter for Huber loss.

        case_weights : ndarray
            Non-negative case weights

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        quadratic : `regreg.identity_quadratic.identity_quadratic` (optional)
            Optional quadratic to be added to objective.

        initial : ndarray
            Initial guess at coefficients.
           
        Returns
        -------

        mglm_obj : `regreg.mglm.mglm`
            General linear model loss.

        """

        losses = [huber_svm(response.shape,
                            response,
                            smoothing_parameter) 
                  for response in successes]

        return klass(losses,
                     offset=offset,
                     quadratic=quadratic,
                     initial=initial,
                     case_weights=case_weights)

    @classmethod
    def cox(klass, 
            X, 
            event_times,
            censoring,
            coef=1., 
            offset=None,
            quadratic=None, 
            initial=None,
            case_weights=None):
        """
        Create a loss for a logistic regression model.

        Parameters
        ----------

        X : [ndarray, `regreg.affine.affine_transform`]
            Design matrix

        event_times : ndarray
            Observed times for Cox proportional hazard model.

        censoring : ndarray 
            Censoring indicator for Cox proportional hazard model
            - 1 indicates observation is a failure, 0 a censored observation.

        offset : ndarray (optional)
            Offset to be applied in parameter space before 
            evaluating loss.

        quadratic : `regreg.identity_quadratic.identity_quadratic` (optional)
            Optional quadratic to be added to objective.

        initial : ndarray
            Initial guess at coefficients.
           
        Returns
        -------

        mglm_obj : `regreg.mglm.mglm`
            General linear model loss.

        """

        losses = [cox_loglike(event_times[i].shape,
                              event_times[i],
                              censoring[i],
                              coef=coef)
                  for i in range(len(event_times))]

        return klass(losses,
                     offset=offset,
                     quadratic=quadratic,
                     initial=initial,
                     case_weights=case_weights)

class multinomial_loglike(smooth_atom):

    overflow_clip = 50

    """
    An overparametrized multinomial regression for nominal responses (e.g. Agresti, pg 267)
    """

    objective_template = r"""\ell^{M}\left(%(var)s\right)"""

    def __init__(self, 
                 shape, 
                 counts, 
                 coef=1., 
                 offset=None,
                 initial=None,
                 quadratic=None,
                 case_weights=None):

        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)

        if sparse.issparse(counts):
            #Convert sparse success vector to an array
            self.counts = counts.toarray()
        else:
            self.counts = counts

        self.J = self.counts.shape[1]

        if not np.allclose(np.round(self.counts), self.counts):
            raise ValueError("Counts vector is not integer valued")
        if np.min(self.counts) < 0:
            raise ValueError("Counts vector is not non-negative")

        self.trials = np.sum(self.counts, axis=1)

        saturated = self.counts / (1. * self.trials[:,np.newaxis])
        loss_terms = np.log(saturated) * self.counts
        loss_terms[np.isnan(loss_terms)] = 0
        loss_constant = - coef * loss_terms.sum()

        devq = identity_quadratic(0,0,0,-loss_constant)
        self.quadratic += devq

        self.data = self.counts, self.trials

        if case_weights is not None:
            if not np.all(case_weights >= 0):
                raise ValueError('case_weights should be non-negative')
            self.case_weights = np.asarray(case_weights)
            if self.case_weights.shape != self.shape[:1]:
                raise ValueError('case_weights should have same shape as response')
        else:
            self.case_weights = None

    def get_data(self):
        return self._data

    def set_data(self, data):
        self._data = data

    data = property(get_data, set_data)

    def smooth_objective(self, 
                         natural_param, 
                         mode='both', 
                         check_feasibility=False,
                         case_weights=None):
        """
        Evaluate a smooth function and/or its gradient

        if mode == 'both', return both function value and gradient
        if mode == 'grad', return only the gradient
        if mode == 'func', return only the function value
        """
        eta = self.apply_offset(natural_param)  # shorthand
        eta = np.clip(eta, 
                      -self.overflow_clip, 
                      self.overflow_clip) # to prevent overflow
        exp_eta = np.exp(eta)

        if case_weights is None:
            cw = np.ones(natural_param.shape[0])
        else:
            cw = case_weights

        if mode == 'both':
            ratio = ((self.trials/np.sum(exp_eta, axis=1)) * exp_eta.T).T
            f, g = (- self.scale(np.sum(cw[:,None] * self.counts * eta) -  np.dot(cw * self.trials, np.log(np.sum(exp_eta, axis=1)))),
                      - self.scale(cw[:,None] * (self.counts - ratio)))
            return f, g
        elif mode == 'grad':
            ratio = ((self.trials/np.sum(exp_eta, axis=1)) * exp_eta.T).T
            f, g = None, - self.scale(cw[:,None] * (self.counts - ratio))
            return g
        elif mode == 'func':
            f, g = - self.scale(np.sum(cw[:,None] * self.counts * eta) -  np.dot(cw * self.trials, np.log(np.sum(exp_eta, axis=1)))), None
            return f
        else:
            raise ValueError("mode incorrectly specified")

    def subsample(self, case_idx):
        """
        Create a saturated loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `smooth_atom`
            Loss after discarding all
            cases not in `idx.

        """
        loss_cp = copy(self)
        if loss_cp.case_weights is None:
            case_weights = loss_cp.case_weights = np.ones(self.shape[0])
        else:
            case_weights = loss_cp.case_weights

        idx_bool = np.zeros_like(case_weights, np.bool)
        idx_bool[case_idx] = 1

        case_weights *= idx_bool
        return loss_cp

class multinomial_baseline_loglike(smooth_atom):

    overflow_clip = 50

    """
    A class for baseline-category logistic regression for nominal responses (e.g. Agresti, pg 267)
    """

    objective_template = r"""\ell^{M}\left(%(var)s\right)"""

    def __init__(self, 
                 shape, 
                 counts, 
                 coef=1., 
                 offset=None,
                 initial=None,
                 quadratic=None,
                 case_weights=None):

        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)

        if sparse.issparse(counts):
            #Convert sparse success vector to an array
            self.counts = counts.toarray()
        else:
            self.counts = counts

        self.J = self.counts.shape[1]
        #Select the counts for the first J-1 categories
        self.firstcounts = self.counts[:,range(self.J-1)]

        if not np.allclose(np.round(self.counts),self.counts):
            raise ValueError("Counts vector is not integer valued")
        if np.min(self.counts) < 0:
            raise ValueError("Counts vector is not non-negative")

        self.trials = np.sum(self.counts, axis=1)

        if shape[1] != self.J - 1:
            raise ValueError("Primal shape is incorrect - should only have coefficients for first J-1 categories")

        saturated = self.counts / (1. * self.trials[:,np.newaxis])
        loss_terms = np.log(saturated) * self.counts
        loss_terms[np.isnan(loss_terms)] = 0
        loss_constant = - coef * loss_terms.sum()

        devq = identity_quadratic(0,0,0,-loss_constant)
        self.quadratic += devq

        self.data = self.counts, self.trials

        if case_weights is not None:
            if not np.all(case_weights >= 0):
                raise ValueError('case_weights should be non-negative')
            self.case_weights = np.asarray(case_weights)
            if self.case_weights.shape != self.shape[:1]:
                raise ValueError('case_weights should have same shape as response')
        else:
            self.case_weights = None

    def get_data(self):
        return self._data

    def set_data(self, data):
        self._data = data

    data = property(get_data, set_data)

    def smooth_objective(self, 
                         natural_parameter,
                         mode='both', 
                         check_feasibility=False,
                         case_weights=None):
        """
        Evaluate a smooth function and/or its gradient

        if mode == 'both', return both function value and gradient
        if mode == 'grad', return only the gradient
        if mode == 'func', return only the function value
        """
        eta = self.apply_offset(natural_parameter) # shorthand
        eta = np.clip(eta, 
                      -self.overflow_clip, 
                      self.overflow_clip) # to prevent overflow
        exp_eta = np.exp(eta)

        if case_weights is None:
            cw = np.ones(eta.shape[0])
        else:
            cw = case_weights

        #TODO: Using transposes to scale the rows of a 2d array - should we use an affine_transform to do this?
        #JT: should be able to do this with np.newaxis

        if mode == 'both':
            ratio = ((self.trials/(1. + np.sum(exp_eta, axis=1))) * exp_eta.T).T
            f, g = (- self.scale(np.sum(cw[:,None] * self.firstcounts * eta) -  np.dot(cw * self.trials, np.log(1. + np.sum(exp_eta, axis=1)))), 
                      - self.scale(cw[:,None] * (self.firstcounts - ratio)))
            return f, g
        elif mode == 'grad':
            ratio = ((self.trials/(1. + np.sum(exp_eta, axis=1))) * exp_eta.T).T
            f, g = None, - self.scale(cw[:,None] * (self.firstcounts - ratio))
            return g
        elif mode == 'func':
            f, g = - self.scale(np.sum(cw[:,None] * self.firstcounts * eta) -  np.dot(cw * self.trials, np.log(1. + np.sum(exp_eta, axis=1)))), None
            return f
        else:
            raise ValueError("mode incorrectly specified")

    def subsample(self, case_idx):
        """
        Create a saturated loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `smooth_atom`
            Loss after discarding all
            cases not in `idx.

        """
        loss_cp = copy(self)
        if loss_cp.case_weights is None:
            case_weights = loss_cp.case_weights = np.ones(self.shape[0])
        else:
            case_weights = loss_cp.case_weights

        idx_bool = np.zeros_like(case_weights, np.bool)
        idx_bool[case_idx] = 1

        case_weights *= idx_bool
        return loss_cp




import nose.tools as nt
import numpy as np
import regreg.affine.factored_matrix as FM
from regreg.affine import power_L, todense
from regreg.atoms.projl1_cython import projl1
from regreg.api import identity_quadratic
from regreg.atoms.tests.test_seminorms import all_close
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_class():

    for shape in [(100, 50), (50, 100)]:
        n, p = shape
        X = np.random.standard_normal((n,p))
        fm = FM.factored_matrix(X, min_singular=1.e-2, affine_offset=0)
        fm.X
        fm.copy()

        U = np.random.standard_normal((p, 10))
        V = np.random.standard_normal((n, 20))
        nt.assert_raises(ValueError, FM.factored_matrix, X, -1)

        fm.linear_map(U)
        fm.affine_map(U)
        fm.adjoint_map(V)

@set_seed_for_test()
def test_rankone():
    x1 = np.random.standard_normal(100)
    x2 = np.random.standard_normal(50)
    fm = FM.factored_matrix(np.multiply.outer(x1, x2), min_singular=1.e-2, affine_offset=0)
    fm.X
    fm.copy()

    X = np.random.standard_normal((100, 50))
    X[:,:7] *= 5

    U = np.random.standard_normal((50, 10))
    V = np.random.standard_normal((100, 20))
    nt.assert_raises(ValueError, FM.factored_matrix, X, -1)

    fm.linear_map(U)
    fm.affine_map(U)
    fm.adjoint_map(V)

def test_zero():

    fm = FM.factored_matrix(np.zeros((100, 50)))
    fm.X

@set_seed_for_test()
def test_partial_svd():
    """
    Rank 10 partial SVD
    """

    X = np.random.standard_normal((100, 50))
    X[:,:7] *= 5

    rank = 5
    U, D, VT = FM.partial_svd(X, rank=rank, padding=10, stopping_rule=lambda D: False, tol=1.e-12)
    nt.assert_true(np.linalg.norm(np.dot(U.T, U) - np.identity(rank)) < 1.e-4)
    nt.assert_true(np.linalg.norm(np.dot(VT, VT.T) - np.identity(rank)) < 1.e-4)
    U_np, D_np, VT_np = np.linalg.svd(X, full_matrices=False)
    nt.assert_true(np.linalg.norm(U - np.dot(U, np.dot(U_np.T[:rank], U_np[:,:rank]))) < 1.e-4)
    nt.assert_true(np.linalg.norm(VT - np.dot(VT, np.dot(VT_np[:rank].T, VT_np[:rank]))) < 1.e-4)

    FM.partial_svd(X, rank=rank, padding=10, stopping_rule=lambda D: False, tol=1.e-12,
                   warm_start=np.random.standard_normal(X.shape[0]))

@set_seed_for_test()
def test_stopping_rule():
    '''
    use a stopping rule in compute_iterative_svd
    '''

    def soft_threshold_rule(L):
        return lambda D: np.fabs(D).min() <= L

    L = 30
    svt_rule = soft_threshold_rule(L)

    X = np.random.standard_normal((100, 50))
    X[:,:7] *= 5

    U, D, VT = FM.compute_iterative_svd(X, initial_rank=3, stopping_rule=svt_rule, tol=1.e-12, debug=True)

    D2 = (D - L) * (D > L)
    D1 = np.linalg.svd(X)[1]
    D1 = (D1 - L) * (D1 > L)
    rank = (D2 > 0).sum()
    all_close(D1[:rank], D2[:rank], 'stopping_rule', None)

@set_seed_for_test()
@np.testing.dec.skipif(True, msg="Proximal maps for factored_matrix are not fully worked out")
def test_proximal_maps():

    X = np.random.standard_normal((100, 50))
    X[:,:7] *= 5

    P = FM.nuclear_norm(X.shape, lagrange=1)
    RP = todense(P.lagrange_prox(X))

    B = FM.nuclear_norm(X.shape, bound=1)
    RB = todense(B.bound_prox(X))

    BO = FM.operator_norm(X.shape, bound=1)
    PO = FM.operator_norm(X.shape, lagrange=1)

    RPO = todense(PO.lagrange_prox(X))
    RBO = todense(BO.bound_prox(X))

    D = np.linalg.svd(X, full_matrices=0)[1]
    lD = np.linalg.svd(RP, full_matrices=0)[1]
    lagrange_rank = (lD > 1.e-10).sum()
    all_close(lD[:lagrange_rank] + P.lagrange, D[:lagrange_rank], 'proximal lagrange', None)

    bD = np.linalg.svd(RB, full_matrices=0)[1]
    bound_rank = (bD > 1.e-10).sum()

    all_close(bD[:bound_rank], projl1(D, B.bound)[:bound_rank], 'proximal bound', None)

    nt.assert_true(np.linalg.norm(RPO+RB-X) / np.linalg.norm(X) < 0.01)
    nt.assert_true(np.linalg.norm(RBO+RP-X) / np.linalg.norm(X) < 0.01)

    # running code to ensure it is tested

    P.conjugate
    P.quadratic = identity_quadratic(1, 0, 0, 0)
    P.conjugate

    BO.conjugate
    BO.quadratic = identity_quadratic(1, 0, 0, 0)
    BO.conjugate

    B.conjugate
    B.quadratic = identity_quadratic(1, 0, 0, 0)
    B.conjugate

    PO.conjugate
    PO.quadratic = identity_quadratic(1, 0, 0, 0)
    PO.conjugate

@set_seed_for_test()
@np.testing.dec.skipif(True, msg="Proximal maps for factored_matrix are not fully worked out")
def test_proximal_method():

    X = np.random.standard_normal((100, 50))
    X[:,:7] *= 5

    qX = identity_quadratic(1,X,0,0)
    P = FM.nuclear_norm(X.shape, lagrange=1)
    RP = todense(P.proximal(qX))

    B = FM.nuclear_norm(X.shape, bound=1)
    RB = todense(B.proximal(qX))

    BO = FM.operator_norm(X.shape, bound=1)
    PO = FM.operator_norm(X.shape, lagrange=1)

    RPO = todense(PO.proximal(qX))
    RBO = todense(BO.proximal(qX))

    D = np.linalg.svd(X, full_matrices=0)[1]
    lD = np.linalg.svd(RP, full_matrices=0)[1]
    lagrange_rank = (lD > 1.e-10).sum()
    all_close(lD[:lagrange_rank] + P.lagrange, D[:lagrange_rank], 'proximal method lagrange', None)

    bD = np.linalg.svd(RB, full_matrices=0)[1]
    bound_rank = (bD > 1.e-10).sum()

    all_close(bD[:bound_rank], projl1(D, B.bound)[:bound_rank], 'proximal method bound', None)

    nt.assert_true(np.linalg.norm(RPO+RB-X) / np.linalg.norm(X) < 0.01)
    nt.assert_true(np.linalg.norm(RBO+RP-X) / np.linalg.norm(X) < 0.01)

@set_seed_for_test()
def test_max_rank():

    X = np.random.standard_normal((100, 200))
    nt.assert_raises(ValueError, FM.compute_iterative_svd, X, max_rank=2)

    U, D, VT = FM.compute_iterative_svd(X)
    nt.assert_true(np.linalg.norm(np.dot(U.T, U) - np.identity(100)) < 1.e-6)
    nt.assert_true(np.linalg.norm(np.dot(VT, VT.T) - np.identity(100)) < 1.e-6)

    U, D, VT = FM.compute_iterative_svd(X, max_rank=200)
    nt.assert_true(np.linalg.norm(np.dot(U.T, U) - np.identity(100)) < 1.e-6)
    nt.assert_true(np.linalg.norm(np.dot(VT, VT.T) - np.identity(100)) < 1.e-6)




import numpy as np
try:
    from sklearn.model_selection import cross_validate
    from sklearn.model_selection import GridSearchCV
    from ..api import (probit,
                       probit_lagrange,
                       probit_classifier,
                       probit_classifier_lagrange)
    have_sklearn = True
except ImportError:
    have_sklearn = False

from ...api import l1norm
from ...tests.decorators import set_seed_for_test

@np.testing.dec.skipif(not have_sklearn)
@set_seed_for_test()
def test_probit():

    n, p = 100, 20
    X = np.random.standard_normal((n, p))
    y = np.random.standard_normal(n)
    y = y > 0

    atom = l1norm
    atom_args = {'shape':p, 'bound':3}

    probit_lasso = probit(atom,
                          atom_args)

    probit_lasso.fit(X, y)
    print(cross_validate(probit_lasso, X, y, cv=10))

    # grid search
    params = {'atom_params':[{'shape':p,
                              'bound': b} for b in [3, 4, 5]]}
    lasso_cv = GridSearchCV(probit_lasso, params, cv=3)
    lasso_cv.fit(X, y)

    probit_lasso_offset = probit(atom,
                                 atom_args,
                                 offset=True)
    y_offset = np.array([y, np.random.standard_normal(n)]).T
    probit_lasso_offset.fit(X, y_offset)

    probit_lasso_weights = probit(atom,
                                  atom_args,
                                  case_weights=True,
                                  score_method='accuracy')

    y_weights = np.array([y, np.ones_like(y)]).T
    probit_lasso_weights.fit(X, y_weights)

    np.testing.assert_allclose(probit_lasso_weights._coefs,
                               probit_lasso._coefs)

    GridSearchCV(probit_lasso_offset, params, cv=3).fit(X, y_offset)
    GridSearchCV(probit_lasso_weights, params, cv=3).fit(X, y_weights)

    probit_lasso_both = probit(atom,
                               atom_args,
                               offset=True,
                               case_weights=True,
                               score_method='R2')
    y_both = np.array([y, np.ones_like(y), y_offset[:,1]]).T
    GridSearchCV(probit_lasso_both, params, cv=3).fit(X, y_both)

@np.testing.dec.skipif(not have_sklearn)
@set_seed_for_test()
def test_probit_classifier():

    n, p = 100, 20
    X = np.random.standard_normal((n, p))
    y = np.random.standard_normal(n)
    y = y > 0

    atom = l1norm
    atom_args = {'shape':p, 'bound':3}

    probit_lasso = probit_classifier(atom,
                                     atom_args)

    probit_lasso.fit(X, y)
    print(cross_validate(probit_lasso, X, y, cv=10))

    # grid search
    params = {'atom_params':[{'shape':p,
                              'bound': b} for b in [3, 4, 5]]}
    lasso_cv = GridSearchCV(probit_lasso, params, cv=3)
    lasso_cv.fit(X, y)

    probit_lasso_weights = probit_classifier(atom,
                                             atom_args,
                                             case_weights=True)

    y_weights = np.array([y, np.ones_like(y)]).T
    probit_lasso_weights.fit(X, y_weights)

    np.testing.assert_allclose(probit_lasso_weights._coefs,
                               probit_lasso._coefs)

    GridSearchCV(probit_lasso_weights, params, cv=3).fit(X, y_weights)







import itertools

import numpy as np

import regreg.api as rr
import regreg.atoms.cones as C
import regreg.atoms.svd_norms as C_SVD
from regreg.tests.decorators import set_seed_for_test

import nose.tools as nt

from .test_seminorms import Solver, SolverFactory


class ConeSolverFactory(SolverFactory):

    FISTA_choices = [True]
    L_choices = [0.3]
    coef_stop_choices = [False]

    def __init__(self, klass):
        self.klass = klass

    def __iter__(self):
        for offset, FISTA, coef_stop, L, q in itertools.product(self.offset_choices,
                                                                self.FISTA_choices,
                                                                self.coef_stop_choices,
                                                                self.L_choices,
                                                                self.quadratic_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            atom = self.klass(self.shape)

            # make sure certain lines of code are tested
            assert(atom == atom)
            atom.latexify(), atom.dual, atom.conjugate

            if q: 
                atom.quadratic = rr.identity_quadratic(0,0,np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = Solver(atom, interactive=self.interactive, 
                            coef_stop=coef_stop,
                            FISTA=FISTA,
                            L=L)
            yield solver


@set_seed_for_test()
@np.testing.dec.slow
def test_proximal_maps():
    for klass in sorted(C.conjugate_cone_pairs.keys(), key=str):
        if klass in [C_SVD.nuclear_norm_epigraph,
                     C_SVD.nuclear_norm_epigraph_polar,
                     C_SVD.operator_norm_epigraph,
                     C_SVD.operator_norm_epigraph_polar]:
            ConeSolverFactory.shape = (5,4)
        else:
            ConeSolverFactory.shape = 20
        factory = ConeSolverFactory(klass)
        for solver in factory:
            for t in solver.all_tests():
                yield t



from copy import copy

import numpy as np
from scipy import sparse
from scipy.stats import norm as normal_dbn

from ..identity_quadratic import identity_quadratic
from . import smooth_atom

class logistic_loglike(smooth_atom):

    """
    A class for combining the logistic log-likelihood with a general seminorm
    """

    objective_template = r"""\ell^{\text{logit}}\left(%(var)s\right)"""

    def __init__(self, 
                 shape, 
                 successes, 
                 trials=None, 
                 coef=1., 
                 offset=None,
                 quadratic=None,
                 initial=None,
                 case_weights=None):

        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)

        self.data = (successes, trials)

        saturated = self.successes / self.trials

        _mask = (saturated != 0) * (saturated != 1)
        loss_terms = (np.log(saturated[_mask]) * self.successes[_mask] +
                      np.log(1 - saturated[_mask]) *
                      ((self.trials - self.successes)[_mask]))
        loss_constant = -coef * loss_terms.sum()

        devq = identity_quadratic(0,0,0,-loss_constant)
        self.quadratic += devq

        if case_weights is not None:
            if not np.all(case_weights >= 0):
                raise ValueError('case_weights should be non-negative')
            self.case_weights = np.asarray(case_weights)
            if self.case_weights.shape != self.successes.shape:
                raise ValueError('case_weights should have same shape as successes')
        else:
            self.case_weights = None

    def smooth_objective(self, 
                         natural_param, 
                         mode='both', 
                         check_feasibility=False,
                         case_weights=None):
        """

        Evaluate the smooth objective, computing its value, gradient or both.

        Parameters
        ----------

        natural_param : ndarray
            The current parameter values.

        mode : str
            One of ['func', 'grad', 'both']. 

        check_feasibility : bool
            If True, return `np.inf` when
            point is not feasible, i.e. when `natural_param` is not
            in the domain.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        If `mode` is 'func' returns just the objective value 
        at `natural_param`, else if `mode` is 'grad' returns the gradient
        else returns both.
        """
        
        if case_weights is None:
            case_weights = np.ones_like(natural_param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        x = natural_param # shorthand

        #Check for overflow in np.exp (can occur during initial backtracking steps)
        x = self.apply_offset(x)
        if np.max(x) > 1e2:
            overflow = True
            not_overflow_ind = np.where(x <= 1e2)[0]
            exp_x = np.exp(x[not_overflow_ind])
        else:
            overflow = False
            exp_x = np.exp(x)
            
        if mode == 'both':
            ratio = self.trials * 1.
            if overflow:
                log_exp_x = x * 1.
                log_exp_x[not_overflow_ind] = np.log(1.+exp_x)
                ratio[not_overflow_ind] *= exp_x/(1.+exp_x)
            else:
                log_exp_x = np.log(1.+exp_x)
                ratio *= exp_x/(1.+exp_x)
                
            f, g = -self.scale((np.dot(cw * self.successes, x) - 
                                np.sum(cw * self.trials * log_exp_x))), - self.scale(cw * (self.successes - ratio))
            return f, g
        elif mode == 'grad':
            ratio = self.trials * 1.
            if overflow:
                ratio[not_overflow_ind] *= exp_x/(1.+exp_x)
            else:
                ratio *= exp_x/(1.+exp_x)
            f, g = None, - self.scale(cw * (self.successes - ratio))
            return g

        elif mode == 'func':
            if overflow:
                log_exp_x = x * 1.
                log_exp_x[not_overflow_ind] = np.log(1.+exp_x)
            else:
                log_exp_x = np.log(1.+exp_x)
            f, g = (- self.scale(np.dot(cw * self.successes, x) - 
                                 np.sum(cw * self.trials * log_exp_x)), None)
            return f
        else:
            raise ValueError("mode incorrectly specified")

    # Begin loss API

    def hessian(self, natural_param, case_weights=None):
        """
        Hessian of the loss.

        Parameters
        ----------

        natural_param : ndarray
            Parameters where Hessian will be evaluated.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        hess : ndarray
            A 1D-array representing the diagonal of the Hessian
            evaluated at `natural_param`.
        """

        x = natural_param # shorthand

        if case_weights is None:
            case_weights = np.ones_like(natural_param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        if np.max(x) > 1e2:
            overflow = True
            not_overflow_ind = np.where(x <= 1e2)[0]
            exp_x = np.zeros_like(x)
            exp_x[not_overflow_ind] = np.exp(x[not_overflow_ind])
            exp_x[~not_overflow_ind] = np.exp(100)
        else:
            overflow = False
            exp_x = np.exp(x)

        return self.scale(exp_x / (1 + exp_x)**2 * self.trials * cw)

    def get_data(self):
        return self.successes

    def set_data(self, data):
        if type(data) == type((3,)):
            successes, trials = data
        else:
            successes = data
            trials = None

        if sparse.issparse(successes):
            #Convert sparse success vector to an array
            self.successes = successes.toarray().flatten() * 1.
        else:
            self.successes = np.asarray(successes).astype(np.float)

        if trials is None and hasattr(self, 'trials') and self.trials is not None:
            trials = self.trials

        if trials is None:
            if not set([0,1]).issuperset(np.unique(self.successes)):
                raise ValueError("Number of successes is not binary - must specify number of trials")
            self.trials = np.ones_like(self.successes)
        else:
            if np.min(trials-self.successes) < 0:
                raise ValueError("Number of successes greater than number of trials")
            if np.min(self.successes) < 0:
                raise ValueError("Response coded as negative number - should be non-negative number of successes")
            self.trials = trials * 1.

    data = property(get_data, set_data)

    def __copy__(self):
        successes, trials = self.data, self.trials
        return logistic_loglike(self.shape,
                                copy(successes),
                                trials=copy(trials),
                                coef=self.coef,
                                offset=copy(self.offset),
                                quadratic=copy(self.quadratic),
                                initial=copy(self.coefs),
                                case_weights=copy(self.case_weights))

    def subsample(self, case_idx):
        """
        Create a saturated loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `smooth_atom`
            Loss after discarding all
            cases not in `idx.

        """
        loss_cp = copy(self)
        if loss_cp.case_weights is None:
            case_weights = loss_cp.case_weights = np.ones(self.shape[0])
        else:
            case_weights = loss_cp.case_weights

        idx_bool = np.zeros_like(case_weights, np.bool)
        idx_bool[case_idx] = 1

        case_weights *= idx_bool
        return loss_cp

    # End loss API

    def mean_function(self, eta, trials=None):
        _exp_eta = np.exp(eta)
        return _exp_eta / (1. + _exp_eta)

class probit_loglike(smooth_atom):

    """
    A class for combining the logistic log-likelihood with a general seminorm
    """

    objective_template = r"""\ell^{\text{probit}}\left(%(var)s\right)"""

    def __init__(self, 
                 shape, 
                 successes, 
                 trials=None, 
                 coef=1., 
                 offset=None,
                 quadratic=None,
                 initial=None,
                 case_weights=None):

        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)

        self.data = (successes, trials)

        saturated = self.successes / self.trials

        _mask = (saturated != 0) * (saturated != 1)
        loss_terms = (np.log(saturated[_mask]) * self.successes[_mask] +
                      np.log(1 - saturated[_mask]) *
                      ((self.trials - self.successes)[_mask]))
        loss_constant = -coef * loss_terms.sum()

        devq = identity_quadratic(0,0,0,-loss_constant)
        self.quadratic += devq

        if case_weights is not None:
            if not np.all(case_weights >= 0):
                raise ValueError('case_weights should be non-negative')
            self.case_weights = np.asarray(case_weights)
            if self.case_weights.shape != self.successes.shape:
                raise ValueError('case_weights should have same shape as successes')
        else:
            self.case_weights = None

    def smooth_objective(self, 
                         natural_param, 
                         mode='both', 
                         check_feasibility=False,
                         case_weights=None):
        """

        Evaluate the smooth objective, computing its value, gradient or both.

        Parameters
        ----------

        natural_param : ndarray
            The current parameter values.

        mode : str
            One of ['func', 'grad', 'both']. 

        check_feasibility : bool
            If True, return `np.inf` when
            point is not feasible, i.e. when `natural_param` is not
            in the domain.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        If `mode` is 'func' returns just the objective value 
        at `natural_param`, else if `mode` is 'grad' returns the gradient
        else returns both.
        """
        
        if case_weights is None:
            case_weights = np.ones_like(natural_param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        x = natural_param # shorthand


        prob = normal_dbn.cdf(x)
        prob[x < -20] = normal_dbn.cdf(-20)   # to prevent overflow
        prob_c = normal_dbn.sf(x) # to prevent overflow
        prob_c[x > 20] = normal_dbn.sf(20)

        if mode == 'both':

            lprob = np.log(prob)
            lprob_c = np.log(prob_c)
            pdf = normal_dbn.pdf(x)

            f, g = (-self.scale(np.sum(cw * (self.successes * lprob + 
                                            (self.trials - self.successes) * lprob_c))),
                     -self.scale(cw * pdf * (self.successes / prob - 
                                             (self.trials - self.successes) / prob_c)))
            return f, g
        elif mode == 'grad':
            pdf = normal_dbn.pdf(x)
            g = - self.scale(cw * pdf * (self.successes / prob - 
                                         (self.trials - self.successes) / prob_c)) 
            return g

        elif mode == 'func':

            lprob = np.log(prob)
            lprob_c = np.log(prob_c)

            f = -self.scale(np.sum(cw * (self.successes * lprob + 
                                         (self.trials - self.successes) * lprob_c)))
            return f
        else:
            raise ValueError("mode incorrectly specified")

    # Begin loss API

    def hessian(self, natural_param, case_weights=None):
        """
        Hessian of the loss.

        Parameters
        ----------

        natural_param : ndarray
            Parameters where Hessian will be evaluated.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        hess : ndarray
            A 1D-array representing the diagonal of the Hessian
            evaluated at `natural_param`.
        """

        x = natural_param # shorthand

        if case_weights is None:
            case_weights = np.ones_like(natural_param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        pi = self.mean_function(x)
        V = pi * (1 - pi)
        deriv_term = normal_dbn.pdf(x)**2
        return self.scale(deriv_term * self.trials * cw / V)

    def get_data(self):
        return self.successes

    def set_data(self, data):
        if type(data) == type((3,)):
            successes, trials = data
        else:
            successes = data
            trials = None

        if sparse.issparse(successes):
            #Convert sparse success vector to an array
            self.successes = successes.toarray().flatten() * 1.
        else:
            self.successes = np.asarray(successes).astype(np.float)

        if trials is None and hasattr(self, 'trials') and self.trials is not None:
            trials = self.trials

        if trials is None:
            if not set([0,1]).issuperset(np.unique(self.successes)):
                raise ValueError("Number of successes is not binary - must specify number of trials")
            self.trials = np.ones_like(self.successes)
        else:
            if np.min(trials-self.successes) < 0:
                raise ValueError("Number of successes greater than number of trials")
            if np.min(self.successes) < 0:
                raise ValueError("Response coded as negative number - should be non-negative number of successes")
            self.trials = trials * 1.

    data = property(get_data, set_data)

    def __copy__(self):
        successes, trials = self.data, self.trials
        return probit_loglike(self.shape,
                              copy(successes),
                              trials=copy(trials),
                              coef=self.coef,
                              offset=copy(self.offset),
                              quadratic=copy(self.quadratic),
                              initial=copy(self.coefs),
                              case_weights=copy(self.case_weights))

    def subsample(self, case_idx):
        """
        Create a saturated loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `smooth_atom`
            Loss after discarding all
            cases not in `idx.

        """
        loss_cp = copy(self)
        if loss_cp.case_weights is None:
            case_weights = loss_cp.case_weights = np.ones(self.shape[0])
        else:
            case_weights = loss_cp.case_weights

        idx_bool = np.zeros_like(case_weights, np.bool)
        idx_bool[case_idx] = 1

        case_weights *= idx_bool
        return loss_cp

    # End loss API

    def mean_function(self, eta, trials=None):
        return normal_dbn.cdf(eta)

class cloglog_loglike(smooth_atom):

    """
    A class for combining the logistic log-likelihood with a general seminorm
    """

    objective_template = r"""\ell^{\text{cloglog}}\left(%(var)s\right)"""

    def __init__(self, 
                 shape, 
                 successes, 
                 trials=None, 
                 coef=1., 
                 offset=None,
                 quadratic=None,
                 initial=None,
                 case_weights=None):

        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)

        self.data = (successes, trials)

        saturated = self.successes / self.trials

        _mask = (saturated != 0) * (saturated != 1)
        loss_terms = (np.log(saturated[_mask]) * self.successes[_mask] +
                      np.log(1 - saturated[_mask]) *
                      ((self.trials - self.successes)[_mask]))
        loss_constant = -coef * loss_terms.sum()

        devq = identity_quadratic(0,0,0,-loss_constant)
        self.quadratic += devq

        if case_weights is not None:
            if not np.all(case_weights >= 0):
                raise ValueError('case_weights should be non-negative')
            self.case_weights = np.asarray(case_weights)
            if self.case_weights.shape != self.successes.shape:
                raise ValueError('case_weights should have same shape as successes')
        else:
            self.case_weights = None

    def smooth_objective(self, 
                         natural_param, 
                         mode='both', 
                         check_feasibility=False,
                         case_weights=None):
        """

        Evaluate the smooth objective, computing its value, gradient or both.

        Parameters
        ----------

        natural_param : ndarray
            The current parameter values.

        mode : str
            One of ['func', 'grad', 'both']. 

        check_feasibility : bool
            If True, return `np.inf` when
            point is not feasible, i.e. when `natural_param` is not
            in the domain.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        If `mode` is 'func' returns just the objective value 
        at `natural_param`, else if `mode` is 'grad' returns the gradient
        else returns both.
        """
        
        if case_weights is None:
            case_weights = np.ones_like(natural_param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        x = natural_param # shorthand

        cutoff = 5
        x_ = np.clip(x, -cutoff, cutoff) # to prevent overflow
        prob_ = 1 - np.exp(-np.exp(x_))
        prob_c_ = np.exp(-np.exp(x_)) 
        prob = prob_.copy()
        prob_c = prob_c_.copy()
        if mode == 'both':

            lprob = np.log(prob)
            lprob_c = np.log(prob_c)
            lpdf = x - np.exp(x)
            pdf = np.exp(lpdf)

            f, g = (-self.scale(np.sum(cw * (self.successes * lprob + 
                                            (self.trials - self.successes) * lprob_c))),
                     -self.scale(cw * pdf * (self.successes / prob - 
                                             (self.trials - self.successes) / prob_c)))
            return f, g
        elif mode == 'grad':
            lpdf = x - np.exp(x)
            pdf = np.exp(lpdf)

            g = - self.scale(cw * pdf * (self.successes / prob - 
                                         (self.trials - self.successes) / prob_c)) 
            return g

        elif mode == 'func':

            lprob = np.log(prob)
            lprob_c = np.log(prob_c)

            f = -self.scale(np.sum(cw * (self.successes * lprob + 
                                         (self.trials - self.successes) * lprob_c)))
            return f
        else:
            raise ValueError("mode incorrectly specified")

    # Begin loss API

    def hessian(self, natural_param, case_weights=None):
        """
        Hessian of the loss.

        Parameters
        ----------

        natural_param : ndarray
            Parameters where Hessian will be evaluated.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        hess : ndarray
            A 1D-array representing the diagonal of the Hessian
            evaluated at `natural_param`.
        """

        x = natural_param # shorthand

        if case_weights is None:
            case_weights = np.ones_like(natural_param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        pi = self.mean_function(x)
        V = pi * (1 - pi)
        lpdf = x - np.exp(x)
        pdf = np.exp(lpdf)
        deriv_term = pdf**2
        return self.scale(deriv_term * self.trials * cw / V)

    def get_data(self):
        return self.successes

    def set_data(self, data):
        if type(data) == type((3,)):
            successes, trials = data
        else:
            successes = data
            trials = None

        if sparse.issparse(successes):
            #Convert sparse success vector to an array
            self.successes = successes.toarray().flatten() * 1.
        else:
            self.successes = np.asarray(successes).astype(np.float)

        if trials is None and hasattr(self, 'trials') and self.trials is not None:
            trials = self.trials

        if trials is None:
            if not set([0,1]).issuperset(np.unique(self.successes)):
                raise ValueError("Number of successes is not binary - must specify number of trials")
            self.trials = np.ones_like(self.successes)
        else:
            if np.min(trials-self.successes) < 0:
                raise ValueError("Number of successes greater than number of trials")
            if np.min(self.successes) < 0:
                raise ValueError("Response coded as negative number - should be non-negative number of successes")
            self.trials = trials * 1.

    data = property(get_data, set_data)

    def __copy__(self):
        successes, trials = self.data, self.trials
        return cloglog_loglike(self.shape,
                               copy(successes),
                               trials=copy(trials),
                               coef=self.coef,
                               offset=copy(self.offset),
                               quadratic=copy(self.quadratic),
                               initial=copy(self.coefs),
                               case_weights=copy(self.case_weights))

    def subsample(self, case_idx):
        """
        Create a saturated loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `smooth_atom`
            Loss after discarding all
            cases not in `idx.

        """
        loss_cp = copy(self)
        if loss_cp.case_weights is None:
            case_weights = loss_cp.case_weights = np.ones(self.shape[0])
        else:
            case_weights = loss_cp.case_weights

        idx_bool = np.zeros_like(case_weights, np.bool)
        idx_bool[case_idx] = 1

        case_weights *= idx_bool
        return loss_cp

    # End loss API

    def mean_function(self, eta, trials=None):
        eta = np.clip(eta, -5, 5)
        return 1 - np.exp(-np.exp(eta))

class huber_svm(smooth_atom):

    objective_template = r"""\ell^{\text{Huber,SVM}}\left(%(var)s\right)"""

    def __init__(self, 
                 shape, 
                 successes, 
                 smoothing_parameter,
                 coef=1., 
                 offset=None,
                 quadratic=None,
                 initial=None,
                 case_weights=None):

        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)

        self.smoothing_parameter = smoothing_parameter
                                         
        if sparse.issparse(successes):
            self.successes = successes.toarray().flatten()
        else:
            self.successes = np.asarray(successes)

        if case_weights is not None:
            if not np.all(case_weights >= 0):
                raise ValueError('case_weights should be non-negative')
            self.case_weights = np.asarray(case_weights)
            if self.case_weights.shape != self.successes.shape:
                raise ValueError('case_weights should have same shape as successes')
        else:
            self.case_weights = None

    def smooth_objective(self, 
                         param, 
                         mode='both', 
                         check_feasibility=False,
                         case_weights=None):
        """

        Evaluate the smooth objective, computing its value, gradient or both.

        Parameters
        ----------

        param : ndarray
            The current parameter values.

        mode : str
            One of ['func', 'grad', 'both']. 

        check_feasibility : bool
            If True, return `np.inf` when
            point is not feasible, i.e. when `param` is not
            in the domain.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        If `mode` is 'func' returns just the objective value 
        at `param`, else if `mode` is 'grad' returns the gradient
        else returns both.
        """
        
        if case_weights is None:
            case_weights = np.ones_like(param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        param = self.apply_offset(param)
        huber_arg = (2 * self.successes - 1) * param # first encode binary as \pm 1
                                                     # then multiply by linpred

        f, g = _huber_svm(huber_arg, smoothing_parameter=self.smoothing_parameter)
        g *= (2 * self.successes - 1) # chain rule for (2 * y - 1) in encoding of y to binary
        if mode == 'func':
            return self.scale((f * cw).sum())
        elif mode == 'grad':
            return self.scale(g * cw)
        elif mode == 'both':
            return self.scale((f * cw).sum()), self.scale(g * cw)
        else:
            raise ValueError("mode incorrectly specified")

    # Begin loss API

    def hessian(self, param, case_weights=None):
        """
        Hessian of the loss.

        Parameters
        ----------

        param : ndarray
            Parameters where Hessian will be evaluated.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        hess : ndarray
            A 1D-array representing the diagonal of the Hessian
            evaluated at `natural_param`.
        """

        # it is piecwise C^2 though... maybe use this?
        raise NotImplementedError('Huber SVM loss is not twice differentiable')

    def get_data(self):
        return self.successes

    def set_data(self, data):
        self.successes = data

    data = property(get_data, set_data)

    def subsample(self, case_idx):
        """
        Create a saturated loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `smooth_atom`
            Loss after discarding all
            cases not in `idx.

        """
        loss_cp = copy(self)
        if loss_cp.case_weights is None:
            case_weights = loss_cp.case_weights = np.ones(self.shape[0])
        else:
            case_weights = loss_cp.case_weights

        idx_bool = np.zeros_like(case_weights, np.bool)
        idx_bool[case_idx] = 1

        case_weights *= idx_bool
        return loss_cp

    def __copy__(self):
        successes = self.data
        return huber_svm(self.shape,
                         copy(successes),
                         self.smoothing_parameter,
                         coef=self.coef,
                         offset=copy(self.offset),
                         quadratic=copy(self.quadratic),
                         initial=copy(self.coefs))
    
    # End loss API

def _huber_svm(z, smoothing_parameter):
    eps = smoothing_parameter
    arg = (1 - z) / eps # function of (2 * y - 1) * eta when y is binary
    proj_arg = (0 < arg) * (arg < 1) * arg + (arg >= 1) 
    return arg * proj_arg - eps * proj_arg**2 / 2, -proj_arg




import numpy as np

def set_seed_for_test(seed=10):
    """
    Fix the seed for random test.

    Parameters
    ----------
    seed : int
        Random seed passed to np.random.seed

    Returns
    -------
    decorator : function
        Decorator which, when applied to a function, sets the 
        random seed before running the test and then
        restores numpy's random state after running the test.

    Notes
    -----
    The decorator itself is decorated with the ``nose.tools.make_decorator``
    function in order to transmit function name, and various other metadata.


    """
    import nose
    def set_seed_decorator(f):

        def setseed_func(*args, **kwargs):
            """Setseed for normal test functions."""
            old_state = np.random.get_state()
            np.random.seed(seed)
            value = f(*args, **kwargs)
            np.random.set_state(old_state)
            return value

        def setseed_gen(*args, **kwargs):
            """Setseed for test generators."""
            old_state = np.random.get_state()
            np.random.seed(seed)
            for x in f(*args, **kwargs):
                yield x
            np.random.set_state(old_state)

        # Choose the right setseed to use when building the actual decorator.
        if nose.util.isgenerator(f):
            setseed = setseed_gen
        else:
            setseed = setseed_func

        return nose.tools.make_decorator(f)(setseed)

    return set_seed_decorator



"""
A collection of commonly used RegReg functions and objects
"""
from __future__ import print_function, division, absolute_import

# Atom imports

from .atoms import affine_atom as linear_atom

from .atoms.seminorms import (l1norm, 
                              l2norm, 
                              supnorm, 
                              positive_part, 
                              constrained_max,
                              constrained_positive_part, 
                              max_positive_part)

from .atoms.cones import (nonnegative, 
                          nonpositive,
                          zero, 
                          zero_constraint, 
                          l1_epigraph, 
                          l1_epigraph_polar,
                          l2_epigraph, 
                          l2_epigraph_polar,
                          linf_epigraph, 
                          linf_epigraph_polar,
                          affine_cone as linear_cone)

from .atoms.block_norms import (l1_l2, 
                                linf_l2, 
                                l1_l1, 
                                linf_linf,
                                block_sum, 
                                block_max)

from .atoms.slope import (slope, 
                          slope_conjugate)

from .atoms.sparse_group_lasso import (sparse_group_lasso,
                                       sparse_group_lasso_dual)

from .atoms.svd_norms import (nuclear_norm, 
                              operator_norm,
                              nuclear_norm_epigraph,
                              nuclear_norm_epigraph_polar,
                              operator_norm_epigraph,
                              operator_norm_epigraph_polar)

from .atoms.linear_constraints import (projection, 
                                       projection_complement)

from .atoms.mixed_lasso import (mixed_lasso, 
                                mixed_lasso_dual)

# group LASSO and variants

from .atoms.group_lasso import (group_lasso, 
                                group_lasso_dual, 
                                group_lasso_epigraph, 
                                group_lasso_epigraph_polar, 
                                group_lasso_dual_epigraph,
                                group_lasso_dual_epigraph_polar)

from .atoms.sparse_group_lasso import (sparse_group_lasso, 
                                       sparse_group_lasso_dual,
                                       inside_set as inside_sparse_group_lasso_ball)

from .atoms.sparse_group_block import (sparse_group_block,
                                       sparse_group_block_dual)

from .atoms.weighted_atoms import (l1norm as weighted_l1norm,
                                   supnorm as weighted_supnorm)

# Affine imports

from .affine import (identity, 
                     selector, 
                     affine_transform, 
                     normalize,
                     linear_transform, 
                     composition as affine_composition,
                     affine_sum, 
                     power_L, 
                     astransform)

from .affine.factored_matrix import (factored_matrix, compute_iterative_svd,
                                     nuclear_norm as factored_nuclear_norm,
                                     operator_norm as factored_operator_norm)

from .affine.block_maps import (block_columns,
                                block_rows)

# Smooth imports

from .smooth import (smooth_atom, 
                     affine_smooth, 
                     sum as smooth_sum)

from .smooth.quadratic import (quadratic_loss, 
                               cholesky, 
                               signal_approximator,
                               squared_error)

from .smooth.glm import (glm, 
                         logistic_loglike, 
                         poisson_loglike, 
                         gaussian_loglike, 
                         cox_loglike,
                         stacked_loglike)

from .smooth.mglm import (multinomial_baseline_loglike,
                          multinomial_loglike,
                          mglm,
                          stacked_common_loglike)

# Problem imports

from .problems.separable import (separable, 
                                 separable_problem)

from .problems.simple import (simple_problem, 
                              gengrad, 
                              nesta, 
                              tfocs)

from .problems.container import container

from .algorithms import FISTA

from .problems.conjugate import conjugate

from .problems.composite import (composite, 
                                 nonsmooth as nonsmooth_composite,
                                 smooth as smooth_composite, 
                                 smooth_conjugate)

from .problems.dual_problem import dual_problem

from .identity_quadratic import identity_quadratic

# sklearn mixin

have_sklearn = True
try:
    from .sklearn_mixin import (sklearn_regression,
                                sklearn_classifier)
except ImportError:
    have_sklearn = False



""" Define distrubution parameters for regreg, including package version

This file contains defines parameters for regreg that we use to fill settings
in setup.py, the regreg top-level docstring, and for building the docs.  In
setup.py in particular, we exec this file, so it cannot import regreg
"""

CLASSIFIERS = ["Development Status :: 3 - Alpha",
               "Environment :: Console",
               "Intended Audience :: Science/Research",
               "License :: OSI Approved :: BSD License",
               "Operating System :: OS Independent",
               "Programming Language :: Python",
               "Topic :: Scientific/Engineering"]

description  = 'A multi-algorithm Python framework for regularized regression'

# Minimum package versions
# Check against requirements.txt and .travis.yml
NUMPY_MIN_VERSION='1.7.1'
SCIPY_MIN_VERSION = '0.9'
CYTHON_MIN_VERSION = '0.18'

NAME                = 'regreg'
MAINTAINER          = "regreg developers"
MAINTAINER_EMAIL    = ""
DESCRIPTION         = description
URL                 = "http://github.org/regreg/regreg"
DOWNLOAD_URL        = ""#"http://github.com/regreg/regreg/archives/master"
LICENSE             = "BSD license"
CLASSIFIERS         = CLASSIFIERS
AUTHOR              = "regreg developers"
AUTHOR_EMAIL        = ""#"regreg-devel@neuroimaging.scipy.org"
PLATFORMS           = "OS Independent"
STATUS              = 'alpha'
PROVIDES            = ["regreg"]
REQUIRES            = ["numpy (>=%s)" % NUMPY_MIN_VERSION,
                       "scipy (>=%s)" % SCIPY_MIN_VERSION]



from copy import copy
import warnings

import numpy as np

from ..identity_quadratic import identity_quadratic
from ..problems.composite import nonsmooth
from ..affine import (linear_transform, identity as identity_transform, 
                    affine_transform)
from ..smooth import affine_smooth
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)


@objective_doc_templater()
class atom(nonsmooth):

    """
    A class that defines the API for support functions.
    """

    objective_vars = nonsmooth.objective_vars.copy()
    objective_vars['klass'] = 'norm'
    objective_vars['dualklass'] = 'dualnorm'

    tol = 1.0e-05

    @doc_template_provider
    def get_conjugate(self):
        """
        Return the conjugate of an given atom.
        Abstract method: subclasses must implement.
        """
        return None
    conjugate = property(get_conjugate, None, None, 'The conjugate of an atom.')

    @doc_template_provider
    def get_dual(self):
        """
        Get the dual of the atom. 
        Abstract method.
        """
        return self.linear_transform, self.conjugate
    dual = property(get_dual)

    @property
    def linear_transform(self):
        """
        The linear transform applied before a penalty is computed. Defaults to regreg.affine.identity

        >>> from regreg.api import l1norm
        >>> penalty = l1norm(30, lagrange=3.4)
        >>> type(penalty.linear_transform)
        <class 'regreg.affine.identity'>

        """
        if not hasattr(self, "_linear_transform"):
            self._linear_transform = identity_transform(self.shape)
        return self._linear_transform

    @doc_template_provider
    def nonsmooth_objective(self, arg, check_feasibility=False):
        """
        The nonsmooth objective function of the atom.
        Includes the quadratic term of the atom.

        Abstract method: subclasses must implement.
        """
        raise NotImplementedError

    def smoothed(self, smoothing_quadratic):
        '''
        Add quadratic smoothing term
        '''
        conjugate_atom = copy(self.conjugate)
        sq = smoothing_quadratic
        if sq.coef in [None, 0]:
            raise ValueError('quadratic term of ' 
                             + 'smoothing_quadratic must be non 0')
        total_q = sq

        if conjugate_atom.quadratic is not None:
            total_q = sq + conjugate_atom.quadratic
        conjugate_atom.quadratic = total_q
        smoothed_atom = conjugate_atom.conjugate
        return smoothed_atom

    @doc_template_provider
    def proximal(self, proxq, prox_control=None):
        r"""
        The proximal operator. 
        Abstract method -- subclasses must implement.
        """
        raise NotImplementedError

class affine_atom(object):
    r"""
    Given a seminorm on :math:`\mathbb{R}^p`, i.e.  :math:`\beta \mapsto
    h_K(\beta)` this class creates a new seminorm that evaluates
    :math:`h_K(D\beta+\alpha)`

    This class does not have a prox, but its dual does. The prox of the dual is

    .. math::

       \text{minimize} \frac{1}{2} \|y-x\|^2_2 + x^T\alpha
       \ \text{s.t.} \ x \in \lambda K

    """

    objective_vars = {'linear':'X'}

    def __init__(self, atom_obj, atransform):
        self.atom = copy(atom_obj)
        # if the affine transform has an offset, put it into
        # the atom. in this way, the affine_transform is actually
        # always linear
        if atransform.affine_offset is not None:
            if self.atom.offset is not None:
                self.atom.offset += atransform.affine_offset
            else:
                self.atom.offset = atransform.affine_offset
            ltransform = linear_transform(atransform.linear_operator,
                                          diag=atransform.diagD)
        else:
            ltransform = atransform
        self.linear_transform = ltransform
        self.input_shape = self.linear_transform.input_shape
        self.output_shape = self.linear_transform.output_shape

    def _repr_latex_(self):
        return r'$$' + self.latexify() + r'$$'

    def latexify(self, var=None, idx=''):
        r'''
        Return a LaTeX representation of an object.

        >>> from regreg.api import l1norm
        >>> penalty = l1norm(10, lagrange=0.9)
        >>> penalty.latexify(var=r'\gamma') 
        '\\lambda_{} \\|\\gamma\\|_1'

        Parameters
        ----------

        var : `string`
            Argument of the functions

        idx : `string`
            Optional subscript index.

        Returns
        -------

        L : `string`
            A LaTeX representation of the atom.

        '''
        template_dict = self.atom.objective_vars.copy()
        template_dict['linear'] = self.objective_vars['linear']
        if var is not None:
            template_dict['var'] = var
        template_dict['idx'] = idx
        return self.atom.latexify(var='%(linear)s_{%(idx)s}%(var)s' % template_dict, idx=idx)

    def __repr__(self):
        return "affine_atom(%s, %s)" % (repr(self.atom),
                                        repr(self.linear_transform.linear_operator))

    @property
    def dual(self):
        tmpatom = copy(self.atom)
        tmpatom.shape = self.output_shape
        return self.linear_transform, tmpatom.conjugate

    def nonsmooth_objective(self, arg, check_feasibility=False):
        """
        Return self.atom.seminorm(self.linear_transform.linear_map(x))
        """
        return self.atom.nonsmooth_objective( \
            self.linear_transform.linear_map(arg),
            check_feasibility=check_feasibility)

    def smoothed(self, smoothing_quadratic):
        '''
        Add quadratic smoothing term
        '''
        ltransform, conjugate_atom = self.dual
        if conjugate_atom.quadratic is not None:
            total_q = smoothing_quadratic + conjugate_atom.quadratic
        else:
            total_q = smoothing_quadratic
        if total_q.coef in [None, 0]:
            raise ValueError('quadratic term of '
                             + 'smoothing_quadratic must be non 0')
        conjugate_atom.quadratic = total_q
        smoothed_atom = conjugate_atom.conjugate
        value = affine_smooth(smoothed_atom, ltransform)
        value.total_quadratic = (smoothed_atom.smoothing_quadratic +
                                 smoothed_atom.atom.quadratic)
        return value

def _work_out_conjugate(offset, quadratic):
    """
    Compute the linear term in the conjugate as well as the offset
    based on having a given offset and the constant and linear
    terms in quadratic.

    """
    if offset is None:
        offset = 0
    else:
        offset = offset
    if quadratic.linear_term is not None:
        linear_term = quadratic.linear_term
    else:
        linear_term = 0
    outq = identity_quadratic(0,0,offset, \
          -quadratic.constant_term - 
          np.sum(offset * linear_term))

    if quadratic.linear_term is not None:
        outoffset = linear_term
    else:
        outoffset = None
    return outoffset, outq




from . import default_lagrange_sequence, warm_start, strong_rules

from .mixed_lasso import mixed_lasso_path as mixed_lasso

from .basil import  basil
from .cross_validate import cross_validate

from .lasso import lasso_path as lasso
from .group_lasso import group_lasso_path as group_lasso
from .sparse_group_lasso import sparse_group_lasso_path as sparse_group_lasso
from .sparse_group_block import sparse_group_block_path as sparse_group_block




"""
Projection onto simplex (and its conjugate)
"""
import numpy as np
from copy import copy

from ..atoms import atom, _work_out_conjugate, affine_atom
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import doc_template_user

from .piecewise_linear import find_solution_piecewise_linear_uncon

class simplex_constraint(atom):

    tol = 1.0e-05

    objective_vars = atom.objective_vars.copy()
    objective_vars['klass'] = 'simplex_constraint'
    objective_vars['dualklass'] = 'maximum'
    objective_vars['initargs'] = '(4,)'

    def __eq__(self, other):
        if self.__class__ == other.__class__:
            return self.shape == other.shape
        return False

    def __copy__(self):
        return self.__class__(copy(self.shape),
                              offset=copy(self.offset),
                              quadratic=self.quadratic)
    
    def __repr__(self):
        if self.quadratic.iszero:
            return "%s(%s, offset=%s)" % \
                (self.__class__.__name__,
                 repr(self.shape), 
                 repr(self.offset))
        else:
            return "%s(%s, offset=%s, quadratic=%s)" % \
                (self.__class__.__name__,
                 repr(self.shape), 
                 repr(self.offset),
                 self.quadratic)

    @doc_template_user
    def get_conjugate(self):
        if self.quadratic.coef == 0:

            offset, outq = _work_out_conjugate(self.offset, self.quadratic)

            cls = conjugate_simplex_pairs[self.__class__]
            atom = cls(self.shape, 
                       offset=offset,
                       quadratic=outq)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    @classmethod
    @doc_template_user
    def linear(cls, linear_operator, diag=False,
               linear_term=None, offset=None):
        l = linear_transform(linear_operator, diag=diag)
        atom = cls(l.input_shape, 
                   linear_term=linear_term, offset=offset)
        return affine_atom(atom, l)

    @doc_template_user
    def proximal(self, proxq, prox_control=None):
        r"""
        Projection onto the simplex.

        """
        offset, totalq = (self.quadratic + proxq).recenter(self.offset)
        if totalq.coef == 0:
            raise ValueError('lipschitz + quadratic coef must be positive')

        prox_arg = -totalq.linear_term / totalq.coef

        eta = self._basic_prox(prox_arg, totalq.coef)
        if offset is None:
            return eta
        else:
            return eta + offset

    def _basic_prox(self, prox_arg, lipschitz):
        # lipschitz is ignored because it is a constraint
        value = find_solution_piecewise_linear_uncon(1, prox_arg)
        return (prox_arg - value).clip(min=0)        

    @doc_template_user
    def constraint(self, arg):
        proj = self._basic_prox(arg, 1)
        in_simplex = (np.linalg.norm(arg - proj)
                      / max([np.linalg.norm(arg) ,1]) < self.tol)
        if in_simplex:
            return 0
        return np.inf

    @doc_template_user
    def nonsmooth_objective(self, arg, check_feasibility=False):
        """
        Value of the simplex constraint.

        Always 0 unless `check_feasibility` is True.

        Parameters
        ----------

        arg : `np.ndarray(np.float)`
            Argument of the seminorm.

        check_feasibility : `bool`
            If `True`, then return `np.inf` if appropriate.

        Returns
        -------

        value : `np.float`
            The seminorm of `arg`.

        """
        arg = np.asarray(arg)
        x_offset = self.apply_offset(arg)
        if check_feasibility:
            value = self.constraint(x_offset)
        else:
            value = 0
        value += self.quadratic.objective(arg, 'func')
        return value

class maximum(atom):

    objective_vars = atom.objective_vars.copy()
    objective_vars['klass'] = 'maximum'
    objective_vars['dualklass'] = 'simplex_constraint'
    objective_vars['initargs'] = '(4,)'

    """
    The conjugate of the simplex constraint: `x.max()`.

    """

    tol = 1.0e-05

    objective_vars = atom.objective_vars.copy()
    objective_vars['klass'] = 'simplex_constraint'
    objective_vars['dualklass'] = 'maximum'
    objective_vars['initargs'] = '(4,)'

    def __eq__(self, other):
        if self.__class__ == other.__class__:
            return self.shape == other.shape
        return False

    def __copy__(self):
        return self.__class__(copy(self.shape),
                              offset=copy(self.offset),
                              quadratic=self.quadratic)
    
    def __repr__(self):
        if self.quadratic.iszero:
            return "%s(%s, offset=%s)" % \
                (self.__class__.__name__,
                 repr(self.shape), 
                 repr(self.offset))
        else:
            return "%s(%s, offset=%s, quadratic=%s)" % \
                (self.__class__.__name__,
                 repr(self.shape), 
                 repr(self.offset),
                 self.quadratic)

    @doc_template_user
    def get_conjugate(self):
        if self.quadratic.coef == 0:

            offset, outq = _work_out_conjugate(self.offset, self.quadratic)

            cls = conjugate_simplex_pairs[self.__class__]
            atom = cls(self.shape, 
                       offset=offset,
                       quadratic=outq)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    @classmethod
    @doc_template_user
    def linear(cls, linear_operator, diag=False,
               linear_term=None, offset=None):
        l = linear_transform(linear_operator, diag=diag)
        atom = cls(l.input_shape, 
                   linear_term=linear_term, offset=offset)
        return affine_atom(atom, l)

    @doc_template_user
    def proximal(self, proxq, prox_control=None):
        r"""
        Projection onto the simplex.

        """
        offset, totalq = (self.quadratic + proxq).recenter(self.offset)
        if totalq.coef == 0:
            raise ValueError('lipschitz + quadratic coef must be positive')

        prox_arg = -totalq.linear_term / totalq.coef
        eta = self._basic_prox(prox_arg, totalq.coef)
        if offset is None:
            return eta
        else:
            return eta + offset

    def _basic_prox(self, prox_arg, lipschitz):
        value = find_solution_piecewise_linear_uncon(1 / lipschitz, prox_arg)
        proj_simplex = (prox_arg - value).clip(min=0)
        return prox_arg - proj_simplex
    
    @doc_template_user
    def constraint(self, x):
        return 0 # no constraints for maximum function

    @doc_template_user
    def nonsmooth_objective(self, arg, check_feasibility=False):
        """
        Value of the simplex constraint.

        Always 0 unless `check_feasibility` is True.

        Parameters
        ----------

        arg : `np.ndarray(np.float)`
            Argument of the seminorm.

        check_feasibility : `bool`
            If `True`, then return `np.inf` if appropriate.

        Returns
        -------

        value : `np.float`
            The seminorm of `arg`.

        """

        arg = np.asarray(arg)
        x_offset = self.apply_offset(arg)
        value = x_offset.max()
        value += self.quadratic.objective(arg, 'func')
        return value

conjugate_simplex_pairs = {}
for n1, n2 in [(simplex_constraint, maximum)]:
    conjugate_simplex_pairs[n1] = n2
    conjugate_simplex_pairs[n2] = n1



from __future__ import print_function, division, absolute_import

import numpy as np
import warnings

from .identity_quadratic import identity_quadratic as sq

class algorithm(object):

    perform_backtrack = True
    step = None
    alpha = 1.1
    debug = False
    max_its=10000
    min_its=5
    default_tol=1e-5
    attempt_decrease = False # start trying to take bigger steps right away

    def __init__(self, composite):
        self.composite = composite

    @property
    def output(self):
        """
        Return the 'interesting' part of the composite arguments.
        In the regression case, this is the tuple (beta, r).
        """
        return self.composite.output

    def fit(self):
        """
        Abstract method.
        """
        raise NotImplementedError


class FISTA(algorithm):

    """
    The FISTA generalized gradient algorithm
    """

    alpha = 1.1 # multiplicative factor for changing step

    def fit(self,
            tol=None,
            min_its=None,
            max_its=None,
            FISTA=True,
            start_step=1.,
            restart=np.inf,
            coef_stop=False,
            return_objective_hist = True,
            monotonicity_restart = True,
            debug = None,
            prox_control={}):

        """
        Use the FISTA (or ISTA) algorithm to fit the problem

        Parameters
        ----------
        FISTA : bool
              use Nesterov weights? If False, this is just gradient descent
        start_step : float
              used in backtracking. This is the starting value of self.step
        restart : int
              Restart Nesterov weights every restart iterations. Default is never (np.inf)
        coef_stop : bool
              Stop based on coefficient changes instead of objective value
        return_objective_hist : bool
              Return the sequence of objective values?
        monotonicity_restart : bool
              If True, Nesterov weights are restarted every time the objective value increases
        debug : bool
              Resets self.debug, which controls whether convergence information is printed
        prox_control : dict
              A dictionary of arguments for fit(), used when the composite.proximal_step itself is a FISTA problem
    
        Returns
        -------

        objective_hist : ndarray
              A vector of objective values. Only return if return_objective_hist is True.

        """

        tol = tol or self.default_tol
        min_its = min_its or self.min_its
        max_its = max_its or self.max_its

        if debug is not None:
            self.debug = debug
        self.prox_control = {}

        objective_hist = np.zeros(max_its)
        
        if self.perform_backtrack and self.step is None:
            #If self.step is not available from last fit use start_step
            self.step = start_step

        self.working_coefs = self.composite.coefs

        # initialize the Nesterov weights
        self.weight_old = 1.

        working_smooth = self.composite.smooth_objective(self.working_coefs, mode='func')
        working_obj = (working_smooth + 
                       self.composite.nonsmooth_objective(self.working_coefs, 
                                                          check_feasibility=True))
        
        if np.isnan(working_obj):
            raise ValueError('objective is NaN')

        itercount = 0
        badstep = 0
        while itercount < max_its:

            #Restart every 'restart' iterations

            if np.mod(itercount+1,restart)==0:
                if self.debug:
                    print("\tRestarting weights")
                self.working_coefs = self.composite.coefs
                t_old = 1.

            objective_hist[itercount] = working_obj

            # Backtracking loop
            if self.perform_backtrack:
                proposed_coefs, proposed_smooth = self.backtrack(itercount)
                     
            else:
                #Use specified Lipschitz constant
                working_grad = self.composite.smooth_objective(self.working_coefs, mode='grad')
                lipschitz = self.composite.lipschitz
                self.step = 1. / lipschitz
                if self.prox_control != {}:
                    proposed_coefs = self.composite.proximal_step(sq(lipschitz, self.working_coefs, working_grad, 0), prox_control=prox_control)
                else:
                    proposed_coefs = self.composite.proximal_step(sq(lipschitz, self.working_coefs, working_grad, 0))
                proposed_smooth = self.composite.smooth_objective(proposed_coefs, mode='func')

            proposed_obj = proposed_smooth + self.composite.nonsmooth_objective(proposed_coefs)

            obj_change = np.fabs(proposed_obj - working_obj)
            obj_rel_change = obj_change/np.max([np.fabs(working_obj),1.])
            if coef_stop:
                coef_rel_change = np.linalg.norm(self.composite.coefs - proposed_coefs) / np.max([1.,np.linalg.norm(proposed_coefs)])

            if self.debug:
                if coef_stop:
                    print(itercount, working_obj,
                          self.step, obj_rel_change,
                          coef_rel_change, tol)
                else:
                    print("%i    obj: %.6e    step: %.2e    "
                          "rel_obj_change: %.2e    tol: %.1e" %
                          (itercount, working_obj,
                           self.step, obj_rel_change,
                           tol))

            if itercount >= min_its:
                if coef_stop:
                    if coef_rel_change < tol:
                        self.working_coefs = proposed_coefs
                        if self.debug:
                            print("Success: Optimization stopped because "
                                  "change in coefficients was below tolerance")
                        break
                else:
                    if obj_rel_change < tol or obj_change < tol:
                        self.working_coefs = proposed_coefs
                        if self.debug:
                            print('Success: Optimization stopped because '
                                  'decrease in objective was below tolerance')
                        break

            self.update_working_coefs(proposed_coefs)

            if itercount > 1 and working_obj < proposed_obj and obj_rel_change > 1e-10 and monotonicity_restart:
                #Adaptive restarting: restart if monotonicity violated
                if self.debug:
                    print("%i Restarting weights" % itercount)
                self.attempt_decrease = True

                if self.weight_old == 1.:
                    #Gradient step didn't decrease objective: tolerance composites or incorrect prox op... time to give up?
                    if self.debug:
                        print("%i  Badstep: current: %f, proposed %f" %
                              (itercount, working_obj, proposed_obj))
                    badstep += 1
                    if badstep > 3:
                        warnings.warn('prox is taking bad steps')
                        if self.debug:
                            print('Caution: Optimization stopped while prox '
                                  'was taking bad steps')
                        break
                itercount += 1
                self.weight_old = 1.
                self.working_coefs = self.composite.coefs

            else:
                self.composite.coefs = proposed_coefs # XXX how much time do we waste on a copy here?
                self.weight_old = self.weight_new
                itercount += 1
                working_obj = proposed_obj

        if self.debug:
            if itercount == max_its:
                print("Optimization stopped because iteration limit was "
                      "reached")
            print("FISTA used", itercount, "of", max_its, "iterations")
        if return_objective_hist:
            return objective_hist[:itercount]

    def update_working_coefs(self, proposed_coefs):
        if FISTA:
            #Use Nesterov weights
            self.weight_new = 0.5 * (1 + np.sqrt(1+4*(self.weight_old**2)))
            #XXX should that be self.working_coefs or self.composite.coefs at the end of this line?
            self.working_coefs = proposed_coefs + ((self.weight_old-1)/(self.weight_new)) * (proposed_coefs - self.composite.coefs)
        else:
            #Just do ISTA
            self.weight_new = 1.
            self.working_coefs = proposed_coefs

    def backtrack(self, itercount):

        if np.mod(itercount+1,100)==0 or self.attempt_decrease:
            self.step *= self.alpha
            self.attempt_decrease = True
        working_smooth, working_grad = self.composite.smooth_objective(self.working_coefs, mode='both')
        while True:
            if self.prox_control != {}:
                proposed_coefs = self.composite.proximal_step(sq(1. / self.step, 
                                                                 self.working_coefs, 
                                                                 working_grad, 
                                                                 0), 
                                                              prox_control=prox_control)
            else:
                proposed_coefs = self.composite.proximal_step(sq(1. / self.step, 
                                                                 self.working_coefs, 
                                                                 working_grad,
                                                                 0))

            proposed_smooth = self.composite.smooth_objective(proposed_coefs, mode='func')

            if not np.isfinite(proposed_smooth):
                stop = False
            elif np.fabs(proposed_smooth - working_smooth)/np.max([1., proposed_smooth]) > 1e-10:
                stop = (proposed_smooth <= (working_smooth + 
                                            np.dot((proposed_coefs - self.working_coefs).reshape(-1),
                                                   working_grad.reshape(-1)) + \
                            (0.5/self.step)*np.linalg.norm(proposed_coefs-self.working_coefs)**2))
            else:
                proposed_grad = self.composite.smooth_objective(proposed_coefs, mode='grad')
                stop = (np.fabs(np.dot((proposed_coefs - self.working_coefs).reshape(-1),
                                       (working_grad - proposed_grad).reshape(-1))) <= 
                        (0.5/self.step)*np.linalg.norm(proposed_coefs-self.working_coefs)**2)
            if stop:
                break

            self.attempt_decrease = False
            self.step /= self.alpha
            if not self.step > 0:
                raise ValueError("stepsize zero reached")
            if self.debug:
                print("%i    Decreasing step to" % itercount, self.step)
        return proposed_coefs, proposed_smooth



import numpy as np

from . import (affine_transform, 
               adjoint,
               astransform)

class block_columns(affine_transform):

    """
    Blockwise transform: mapping where arguments 
    have common number of columns.

    Given a sequence of maps $X_i$ of shape
    $(n_i, q)$ produces an output of shape
    $(\sum_i n_i, q)$

    .. math::
    
       (X_1, \dots, X_k)(\beta_{p \times q) \mapsto 
       \begin{pmatrix} X_1\beta_1 \\ \dots X_k \beta_q 
       \end{pmatrix}

    """

    def __init__(self, transforms):

        '''
        Parameters
        ----------

        transforms : sequence
             A sequence of `affine_transform` whose outputs
             are similar in that their `input_shape` is common
             and their outputs are stackable vertically.

        '''

        self._transforms = [astransform(trans) for trans in transforms]
        if not np.all([trans.input_shape == self._transforms[0].input_shape 
                       for trans in self._transforms]):
            raise ValueError('each block should take same input shape')

        self.input_shape = self._transforms[0].input_shape + (len(self._transforms),)

        self._offsets = [trans.affine_map(np.zeros(trans.input_shape))
                         for trans in self._transforms]
        try:
            ndim = [o.ndim for o in self._offsets]
            if ndim[0] == 1:
                _buffer = np.hstack(self._offsets)
            elif ndim[0] == 2:
                _buffer = np.vstack(self._offsets)
        except ValueError:
            raise ValueError('output of transforms should be stackable')

        self.output_shape = _buffer.shape
        self._slices = []
        idx = 0
        for offset in self._offsets:
            self._slices.append(slice(idx, idx+offset.shape[0], 1))
            idx += offset.shape[0]

    def affine_map(self, arg):
        
        _buffer = np.empty(self.output_shape, np.float)
        for i, info in enumerate(zip(self._slices, 
                                     self._transforms)):
            slice, trans = info
            _buffer[slice] = trans.affine_map(arg[:,i])

        return _buffer

    def linear_map(self, arg):
        
        _buffer = np.empty(self.output_shape, np.float)
        for i, info in enumerate(zip(self._slices, 
                                     self._transforms)):
            slice, trans = info
            _buffer[slice] = trans.linear_map(arg[:,i])

        return _buffer

    def adjoint_map(self, arg):
        
        _adj_buffer = np.empty(self.input_shape, np.float)
        for i, info in enumerate(zip(self._slices, 
                                     self._transforms)):
            slice, trans = info
            _adj_buffer[:,i] = trans.adjoint_map(arg[slice])

        return _adj_buffer

class block_rows(affine_transform):

    """
    Blockwise transform: mapping where arguments 
    have common number of rows.

    Given a sequence of $k$ maps $X_i$ of shape
    $(p, n_i)$ produces an output of shape
    $(p, k)$.

    Formally this is an adjoint of `block_columns`

    .. math::
    
       (Z_1, \dots, Z_k)((Y_1, \dots, Y_k) \mapsto 
       \begin{pmatrix} Z_1 Y_1 & \dots & \dots Z_k Y_k \beta_q 
       \end{pmatrix}

    """

    def __init__(self, transforms):

        '''
        Parameters
        ----------

        transforms : sequence
             A sequence of `affine_transform` whose outputs
             are similar in that their `input_shape` is common
             and their outputs are stackable vertically.

        '''

        self._transforms = [astransform(trans) for trans in transforms]
        # we will use the linear map
        self._adjoint = block_columns([adjoint(trans) 
                                       for trans in self._transforms])
        self._adjointT = self._adjoint.T

        if not np.all([trans.output_shape == self._transforms[0].output_shape 
                       for trans in self._transforms]):
            raise ValueError('each block should produce same output shape')

        self.input_shape = (np.sum([trans.input_shape[0] for trans in self._transforms]),)
        self.output_shape = self._transforms[0].output_shape + (len(self._transforms),)

        _offsets = np.array([trans.affine_map(np.zeros(trans.input_shape))
                             for trans in self._transforms])
        try:
            ndim = [o.ndim for o in _offsets]
            if ndim[0] == 1:
                self._offset = _offsets.T
        except ValueError:
            raise ValueError('output of transforms should be the same and hstackable')

        self.output_shape = self._offset.shape

    def affine_map(self, arg):
        
        return self._adjointT.linear_map(arg) + self._offset

    def linear_map(self, arg):
        
        return self._adjointT.linear_map(arg) 

    def adjoint_map(self, arg):
        
        return self._adjoint.linear_map(arg)




""" Test objective doc templater
"""

from regreg.objdoctemplates import objective_doc_templater

from regreg.doctemplates import (
    doc_templater,
    doc_template_provider,
    doc_template_user)

from nose.tools import assert_equal, assert_raises


def test_obj_doc_templater():
    @objective_doc_templater(dict(b='two'))
    class C(object):
        objective_template = r'''f(%(var)s)'''
        objective_vars = {'var': 'p'}

        @doc_template_user
        @doc_template_provider
        def fa(self, arg):
            """A docstring %(objective)s"""

        @doc_template_user
        @doc_template_provider
        def fb(self, arg):
            """B docstring %(b)s"""

    # Providing a template sets the docstring to None
    assert_equal(C().fa.__doc__, 'A docstring f(p)')
    assert_equal(C().fb.__doc__, "B docstring two")



from itertools import product
import nose.tools as nt

import numpy as np
import scipy.sparse

import regreg.api as rr
import regreg.affine.fused_lasso as FL
from regreg.identity_quadratic import identity_quadratic as sq
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_class():
    p = 50
    for order in range(1,3):
        fused = FL.trend_filter.grid(p, order=order)
        fused2 = FL.trend_filter(np.arange(p), order=order)
        V = np.random.standard_normal(p)
        U = np.random.standard_normal(p - order)
        np.testing.assert_allclose(fused.linear_map(V), fused2.linear_map(V))
        np.testing.assert_allclose(fused.affine_map(V), fused2.affine_map(V))
        np.testing.assert_allclose(fused.adjoint_map(U), fused2.adjoint_map(U))

        V2 = np.random.standard_normal((p, 3))
        U2 = np.random.standard_normal((p - order, 3))

        np.testing.assert_allclose(fused.linear_map(V2), fused2.linear_map(V2))
        np.testing.assert_allclose(fused.affine_map(V2), fused2.affine_map(V2))
        np.testing.assert_allclose(fused.adjoint_map(U2), fused2.adjoint_map(U2))

        if order == 1:
            fusedI = FL.trend_filter_inverse.grid(p, order=order)
            fusedI2 = FL.trend_filter_inverse(np.arange(p), order=order)

            np.testing.assert_allclose(fusedI.linear_map(U), fusedI2.linear_map(U))
            np.testing.assert_allclose(fusedI.affine_map(U), fusedI2.affine_map(U))
            np.testing.assert_allclose(fusedI.adjoint_map(V), fusedI2.adjoint_map(V))

            np.testing.assert_allclose(fusedI.linear_map(U2), fusedI2.linear_map(U2))
            np.testing.assert_allclose(fusedI.affine_map(U2), fusedI2.affine_map(U2))
            np.testing.assert_allclose(fusedI.adjoint_map(V2), fusedI2.adjoint_map(V2))


def test_difference_transform():
    p = 50
    for order in range(1,3):
        FL.difference_transform(np.arange(p), order=order, sorted=False)
        FL.difference_transform(np.arange(p), order=order, transform=False)



from copy import copy
import numpy as np
import nose.tools as nt

import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_conjugate_l1norm():
    '''
    this test verifies that numerically computing the conjugate
    is essentially the same as using the smooth_conjugate
    of the atom
    '''

    q = rr.identity_quadratic(1.2,0,0,0)
    l1 = rr.l1norm(4, lagrange=0.3)
    pen2 = copy(l1)
    pen2.set_quadratic(q)

    v1 = rr.smooth_conjugate(l1, q)
    v2 = rr.conjugate(l1, q, tol=1.e-12, min_its=100)
    v3 = rr.conjugate(pen2, None, tol=1.e-12, min_its=100)
    w = np.random.standard_normal(4)

    u11, u12 = v1.smooth_objective(w)
    u21, u22 = v2.smooth_objective(w)
    u31, u32 = v3.smooth_objective(w)
    np.testing.assert_approx_equal(u11, u21)
    np.testing.assert_allclose(u12, u22, rtol=1.0e-05)
    np.testing.assert_approx_equal(u11, u31)
    np.testing.assert_allclose(u12, u32, rtol=1.0e-05)

    v2.smooth_objective(w, mode='func')
    v2.smooth_objective(w, mode='grad')
    nt.assert_raises(ValueError, v2.smooth_objective, w, 'blah')

@set_seed_for_test()
def test_conjugate_sqerror():
    """
    This verifies the conjugate class can compute the conjugate
    of a quadratic function.
    """

    ridge_coef = 0.4

    X = np.random.standard_normal((10,4))
    Y = np.random.standard_normal(10)
    l = rr.squared_error(X, Y)

    q = rr.identity_quadratic(ridge_coef,0,0,0)
    atom_conj = rr.conjugate(l, q, tol=1.e-12, min_its=100)
    w = np.random.standard_normal(4)
    u11, u12 = atom_conj.smooth_objective(w)

    # check that objective is half of squared error
    np.testing.assert_allclose(l.smooth_objective(w, mode='func'), 0.5 * np.linalg.norm(Y - np.dot(X, w))**2)
    np.testing.assert_allclose(atom_conj.atom.smooth_objective(w, mode='func'), 0.5 * np.linalg.norm(Y - np.dot(X, w))**2)

    XTX = np.dot(X.T, X) 
    XTXi = np.linalg.pinv(XTX)

    quadratic_term = XTX + ridge_coef * np.identity(4)
    linear_term = np.dot(X.T, Y) + w
    b = u22 = np.linalg.solve(quadratic_term, linear_term)
    u21 = (w*u12).sum() - l.smooth_objective(u12, mode='func') - q.objective(u12, mode='func')
    np.testing.assert_allclose(u12, u22, rtol=1.0e-05)
    np.testing.assert_approx_equal(u11, u21)
    



import numpy as np

from regreg.affine.multiscale import multiscale, choose_tuning_parameter
import regreg.api as rr
import regreg.affine as ra
from regreg.tests.decorators import set_seed_for_test

INTERACTIVE = False

if INTERACTIVE:
    try:
        from matplotlib.pyplot import plt
    except ImportError:
        INTERACTIVE = False

def _multiscale_matrix(p, minsize=None):
    minsize = minsize or int(np.around(p**(1/3.)))
    rows = []
    for i in range(p):
        for j in range(i, p):
            if (j - i) >= minsize:
                row = np.zeros(p)
                row[i:j] = 1. / (j-i)
                rows.append(row)
    A = np.array(rows)
    A -= np.mean(A, 1)[:,None]
    return A

@set_seed_for_test()
def test_multiscale():

    M = _multiscale_matrix(200)
    Mtrans = multiscale(200)
    V = np.random.standard_normal(M.shape[1])
    W = np.random.standard_normal(M.shape[0])

    np.testing.assert_allclose(np.dot(M, V), Mtrans.linear_map(V))
    np.testing.assert_allclose(np.dot(M.T, W), Mtrans.adjoint_map(W))
    
    Mtrans.update_slices([(s,e) for s, e in Mtrans.slices])
    np.testing.assert_allclose(np.dot(M, V), Mtrans.linear_map(V))
    np.testing.assert_allclose(np.dot(M.T, W), Mtrans.adjoint_map(W))
    
    Mtrans = multiscale(200, slices=[(s,e) for s, e in Mtrans.slices])
    Mtrans.update_slices(Mtrans.slices)

    Mtrans = multiscale(200, slices=[(s,e) for s, e in Mtrans.slices], scaling=2)
    Mtrans.linear_map(V)
    Mtrans.affine_map(V)
    Mtrans.adjoint_map(W)

@set_seed_for_test()
def test_changepoint():

    p = 150
    M = multiscale(p)
    M.minsize = 10
    X = ra.adjoint(M)

    Y = np.random.standard_normal(p)
    Y[20:50] += 8
    Y += 2
    meanY = Y.mean()

    lammax = np.fabs(X.adjoint_map(Y)).max()

    penalty = rr.l1norm(X.input_shape, lagrange=0.5*lammax)
    loss = rr.squared_error(X, Y - meanY)
    problem = rr.simple_problem(loss, penalty)
    soln = problem.solve()
    Yhat = X.linear_map(soln)
    Yhat += meanY

    if INTERACTIVE:
        plt.scatter(np.arange(p), Y)
        plt.plot(np.arange(p), Yhat)
        plt.show()

@set_seed_for_test()
def test_changepoint_scaled():

    p = 150
    M = multiscale(p)
    M.minsize = 10
    X = ra.adjoint(M)

    Y = np.random.standard_normal(p)
    Y[20:50] += 8
    Y += 2
    meanY = Y.mean()

    lammax = np.fabs(np.sqrt(M.sizes) * X.adjoint_map(Y) / (1 + np.sqrt(np.log(M.sizes)))).max()

    penalty = rr.weighted_l1norm((1 + np.sqrt(np.log(M.sizes))) / np.sqrt(M.sizes), lagrange=0.5*lammax)
    loss = rr.squared_error(X, Y - meanY)
    problem = rr.simple_problem(loss, penalty)
    soln = problem.solve()
    Yhat = X.linear_map(soln)
    Yhat += meanY

    if INTERACTIVE:
        plt.scatter(np.arange(p), Y)
        plt.plot(np.arange(p), Yhat)
        plt.show()

def test_choose_parameter(delta=2, p=60):

    signal = np.zeros(p)
    signal[(p//2):] += delta
    Z = np.random.standard_normal(p) + signal
    p = Z.shape[0]
    M = multiscale(p)
    M.scaling = np.sqrt(M.sizes)
    lam = choose_tuning_parameter(M)
    weights = (lam + np.sqrt(2 * np.log(p / M.sizes))) / np.sqrt(p)

    Z0 = Z - Z.mean()
    loss = rr.squared_error(ra.adjoint(M), Z0)
    penalty = rr.weighted_l1norm(weights, lagrange=1.)
    problem = rr.simple_problem(loss, penalty)
    coef = problem.solve()
    active = coef != 0

    if active.sum():
        X = M.form_matrix(M.slices[active])[0]






import numpy as np, regreg.api as rr, regreg.affine as ra
import nose.tools as nt
import regreg.paths.mixed_lasso as mixed_lasso

from regreg.tests.decorators import set_seed_for_test

from regreg.paths import subsample_columns

def test_subsample_cols():

    X = np.random.standard_normal((50,20)); L = ra.affine_transform(X, None)
    Xs = subsample_columns(L, range(5))
    Xs - X[:,range(5)]
    np.testing.assert_allclose(Xs, X[:,range(5)])

@set_seed_for_test()
def test_path():
    '''
    this test looks at the paths of three different parameterizations
    of the same problem

    '''
    X = np.random.standard_normal((100,5))
    Z = np.zeros((100,10))
    Y = np.random.standard_normal(100)
    U = np.random.standard_normal((100,2))
    betaX = np.array([3,4,5,0,0])
    betaU = np.array([10,-5])
    Y += np.dot(X, betaX) + np.dot(U, betaU)
    Z[:,5:] = -X
    Z[:,:5] = X
    Z2 = np.zeros((100,8))
    Z2[:,:3] = X[:,:3]
    Z2[:,3:6] = -X[:,:3]
    Z2[:,6:] = -X[:,3:]

    lasso1 = mixed_lasso.mixed_lasso_path.gaussian(X, Y, nstep=23)
    lasso2 = mixed_lasso.mixed_lasso_path.gaussian(Z, Y, penalty_structure=[mixed_lasso.POSITIVE_PART]*10, nstep=23)

    sol1 = lasso1.main(inner_tol=1.e-12)
    beta1 = sol1['beta'].todense()

    sol2 = lasso2.main(inner_tol=1.e-12)
    beta2 = sol2['beta'].todense()
    beta2[1:6] = beta2[1:6] - beta2[6:11]
    beta2 = beta2[:6]

    lasso3 = mixed_lasso.mixed_lasso_path.gaussian(Z2,Y, penalty_structure=[mixed_lasso.POSITIVE_PART]*6 + [mixed_lasso.L1_PENALTY]*2, nstep=23)
    sol3 = lasso3.main(inner_tol=1.e-12)
    beta3 = sol3['beta'].todense()
    beta3[1:4] = beta3[1:4] - beta3[4:7]
    beta3[4:6] = - beta3[7:9]
    beta3 = beta3[:6]

    np.testing.assert_allclose(beta1, beta2)
    np.testing.assert_allclose(beta2, beta3)

@set_seed_for_test()
def test_path_unpenalized():
    '''
    this test looks at the paths of three different parameterizations
    of the same problem

    '''
    X = np.random.standard_normal((100,5))
    U = np.random.standard_normal((100,2))
    Z = np.zeros((100,10))
    Y = np.random.standard_normal(100)
    betaX = np.array([3,4,5,0,0])
    betaU = np.array([10,-5])
    Y += np.dot(X, betaX) + np.dot(U, betaU)

    Z[:,5:] = -X
    Z[:,:5] = X
    Z2 = np.zeros((100,8))
    Z2[:,:3] = X[:,:3]
    Z2[:,3:6] = -X[:,:3]
    Z2[:,6:] = -X[:,3:]

    lasso1 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([X,U]),Y, penalty_structure=[mixed_lasso.L1_PENALTY]*5 + [mixed_lasso.UNPENALIZED]*2, nstep=23)
    lasso2 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([Z,U]),Y, penalty_structure=[mixed_lasso.POSITIVE_PART]*10 + [mixed_lasso.UNPENALIZED]*2, nstep=23)

    sol1 = lasso1.main(inner_tol=1.e-12)
    beta1 = sol1['beta'].todense()

    sol2 = lasso2.main(inner_tol=1.e-12)
    beta2 = sol2['beta'].todense()
    beta2[1:6] = beta2[1:6] - beta2[6:11]
    beta2[6:8] = beta2[-2:]
    beta2 = beta2[:8]

    lasso3 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([Z2,U]),Y, 
                                    penalty_structure=[mixed_lasso.POSITIVE_PART]*6 + [mixed_lasso.L1_PENALTY]*2 + [mixed_lasso.UNPENALIZED]*2, nstep=23)
    sol3 = lasso3.main(inner_tol=1.e-12)
    beta3 = sol3['beta'].todense()
    beta3[1:4] = beta3[1:4] - beta3[4:7]
    beta3[4:6] = - beta3[7:9]
    beta3[6:8] = beta3[-2:]
    beta3 = beta3[:8]

    np.testing.assert_allclose(beta1, beta2)
    np.testing.assert_allclose(beta2, beta3)

def test_path_lasso_as_group():
    '''
    this test looks at the paths of three different parameterizations
    of the same problem

    '''
    X = np.random.standard_normal((100,5))
    U = np.random.standard_normal((100,2))
    Z = np.zeros((100,10))
    Y = np.random.standard_normal(100)
    betaX = np.array([3,4,5,0,0])
    betaU = np.array([10,-5])
    Y += np.dot(X, betaX) + np.dot(U, betaU)

    Z[:,5:] = -X
    Z[:,:5] = X
    Z2 = np.zeros((100,8))
    Z2[:,:3] = X[:,:3]
    Z2[:,3:6] = -X[:,:3]
    Z2[:,6:] = -X[:,3:]

    lasso1 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([X,U]),Y, penalty_structure=[mixed_lasso.L1_PENALTY]*5 + [mixed_lasso.UNPENALIZED]*2, nstep=23)
    lasso2 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([Z,U]),Y, penalty_structure=[mixed_lasso.POSITIVE_PART]*10 + [mixed_lasso.UNPENALIZED]*2, nstep=23)

    sol1 = lasso1.main(inner_tol=1.e-12)
    beta1 = np.array(sol1['beta'].todense())

    sol2 = lasso2.main(inner_tol=1.e-12)
    beta2 = np.array(sol2['beta'].todense())
    beta2[1:6] = beta2[1:6] - beta2[6:11]
    beta2[6:8] = beta2[-2:]
    beta2 = beta2[:8]

    lasso3 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([Z2,U]),Y, 
                                    penalty_structure=[mixed_lasso.POSITIVE_PART]*6 + [mixed_lasso.L1_PENALTY]*2 + [mixed_lasso.UNPENALIZED]*2, nstep=23)

    sol3 = lasso3.main(inner_tol=1.e-12)
    beta3 = np.array(sol3['beta'].todense())
    beta3[1:4] = beta3[1:4] - beta3[4:7]
    beta3[4:6] = - beta3[7:9]
    beta3[6:8] = beta3[-2:]
    beta3 = beta3[:8]

    lasso4 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([Z2,U]),Y, 
                                    penalty_structure=[mixed_lasso.POSITIVE_PART]*6 + [0,1] + [mixed_lasso.UNPENALIZED]*2, nstep=23)

    sol4 = lasso4.main(inner_tol=1.e-12)
    beta4 = np.array(sol4['beta'].todense())
    beta4[1:4] = beta4[1:4] - beta4[4:7]
    beta4[4:6] = - beta4[7:9]
    beta4[6:8] = beta4[-2:]
    beta4 = beta4[:8]

    np.testing.assert_allclose(beta1, beta2)
    np.testing.assert_allclose(beta2, beta3)
    np.testing.assert_allclose(beta4, beta3)

def test_path_group_lasso():
    '''
    this test looks at the paths of three different parameterizations
    of the same problem

    '''
    n = 100
    X = np.random.standard_normal((n,10))
    U = np.random.standard_normal((n,2))
    Y = np.random.standard_normal(100)
    betaX = np.array([3,4,5,0,0] + [0]*5)
    betaU = np.array([10,-5])
    Y += (np.dot(X, betaX) + np.dot(U, betaU)) * 5

    Xn = rr.normalize(np.hstack([np.ones((100,1)),X]), inplace=True, center=True, scale=True, intercept_column=0).normalized_array()
    lasso = mixed_lasso.mixed_lasso_path.gaussian(Xn[:,1:] ,Y, penalty_structure=[0]*7+[1]*3, nstep=10)

    sol = lasso.main(inner_tol=1.e-12, verbose=True)
    beta = np.array(sol['beta'].todense())

    sols = []
    sols_sep = []
    for l in sol['lagrange']:
        loss = rr.glm.gaussian(Xn, Y)
        penalty = rr.mixed_lasso([mixed_lasso.UNPENALIZED] + [0]*7 + [1]*3, lagrange=l) # matrix contains an intercept...
        problem = rr.simple_problem(loss, penalty)
        sols.append(problem.solve(tol=1.e-12).copy())

        sep = rr.separable((11,), [rr.l2norm((7,),np.sqrt(7)*l), rr.l2norm((3,),np.sqrt(3)*l)],[np.arange(1,8),np.arange(8,11)])
        sep_problem = rr.simple_problem(loss, sep)
        sols_sep.append(sep_problem.solve(tol=1.e-12).copy())

    sols = np.array(sols).T
    sols_sep = np.array(sols_sep).T

    nt.assert_true(np.linalg.norm(beta - sols) / (1 + np.linalg.norm(beta)) <= 1.e-4)
    nt.assert_true(np.linalg.norm(beta - sols_sep) / (1 + np.linalg.norm(beta)) <= 1.e-4)


def test_path_nonnegative():
    '''
    this test looks at the paths of three different parameterizations
    of the same problem
    '''
    X = np.random.standard_normal((100,5))
    U = np.random.standard_normal((100,2))
    U2 = np.zeros((100,4))
    Y = np.random.standard_normal(100)
    betaX = np.array([3,4,5,0,0])
    betaU = np.array([10,-5])
    Y += np.dot(X, betaX) + np.dot(U, betaU)

    U2[:,:2] = U
    U2[:,2:] = -U

    lasso1 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([X,U]),Y, penalty_structure=[mixed_lasso.L1_PENALTY]*5 + [mixed_lasso.UNPENALIZED]*2, nstep=23)
    lasso2 = mixed_lasso.mixed_lasso_path.gaussian(np.hstack([X,U2]),Y, penalty_structure=[mixed_lasso.L1_PENALTY]*5 + [mixed_lasso.NONNEGATIVE]*4, nstep=23)

    problem = lasso2.problem
    soln = problem.solve(debug=True)

    sol1 = lasso1.main(inner_tol=1.e-12)
    beta1 = sol1['beta'].todense()

    sol2 = lasso2.main(inner_tol=1.e-12)
    beta2 = sol2['beta'].todense()

    beta2[6:8] = beta2[6:8] - beta2[8:10]
    beta2 = beta2[:8]

    nt.assert_true(np.linalg.norm(beta1-beta2) / np.linalg.norm(beta1) < 1.e-3)





from __future__ import print_function, division, absolute_import

import itertools
from copy import copy
import nose.tools as nt

import numpy as np

import regreg.atoms.seminorms as S
import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

def all_close(x, y, msg, solver):
    """
    Check to see if x and y are close. 
    """
    try:
        v = np.linalg.norm(x-y) <= 1.0e-03 * max([1, np.linalg.norm(x), np.linalg.norm(y)])
    except:
        print("""
check_failed
============
msg: %s
x: %s
y: %s
""" % (msg, x, y))
        return False
    v = v or np.allclose(x,y)
    if not v:
        print("""
summary
=======
msg: %s
comparison: %0.3f
x : %s
y : %s
""" % (msg, np.linalg.norm(x-y) / max([1, np.linalg.norm(x), np.linalg.norm(y)]), x, y))
    if not hasattr(solver, 'interactive') or not solver.interactive:
        nt.assert_true(v)
    else:
        print(msg.split('\n')[0])

@set_seed_for_test()
@np.testing.dec.slow
def test_proximal_maps(interactive=False):
    for klass in [S.l1norm,
                  S.supnorm,
                  S.l2norm,
                  S.positive_part,
                  S.constrained_max,
                  S.constrained_positive_part,
                  S.max_positive_part]:
        factory = SolverFactory(klass, 'lagrange')
        for solver in factory:
            penalty = solver.atom
            dual = penalty.conjugate 
            Z = solver.prox_center
            L = solver.L

            yield all_close, penalty.lagrange_prox(Z, lipschitz=L), Z-dual.bound_prox(Z*L)/L, 'testing lagrange_prox and bound_prox starting from atom\n %s ' % klass, None

            # some arguments of the constructor

            nt.assert_raises(AttributeError, setattr, penalty, 'bound', 4.)
            nt.assert_raises(AttributeError, setattr, dual, 'lagrange', 4.)
        
            nt.assert_raises(AttributeError, setattr, penalty, 'bound', 4.)
            nt.assert_raises(AttributeError, setattr, dual, 'lagrange', 4.)

            # call these to ensure coverage at least

            repr(penalty)
            repr(dual)

            penalty.seminorm(Z, lagrange=1)
            penalty.constraint(Z, bound=1)

            dual.seminorm(Z, lagrange=1)
            dual.constraint(Z, bound=1)

            for t in solver.all_tests():
                yield t

        factory = SolverFactory(klass, 'bound')
        for solver in factory:
            for t in solver.all_tests():
                yield t

    for klass in sorted(S.nonpaired_atoms):
        factory = SolverFactory(klass, 'lagrange')
        for solver in factory:

            penalty = solver.atom
            dual = penalty.conjugate 
            Z = solver.prox_center
            L = solver.L

            yield all_close, penalty.lagrange_prox(Z, lipschitz=L), Z-dual.bound_prox(Z*L)/L, 'testing lagrange_prox and bound_prox starting from atom %s\n ' % klass, None

            nt.assert_raises(AttributeError, setattr, penalty, 'bound', 4.)
            nt.assert_raises(AttributeError, setattr, dual, 'lagrange', 4.)
        
            nt.assert_raises(AttributeError, setattr, penalty, 'bound', 4.)
            nt.assert_raises(AttributeError, setattr, dual, 'lagrange', 4.)

            for t in solver.all_tests():
                yield t

class SolverFactory(object):

    offset_choices = [True, False]
    FISTA_choices =[True,False]
    coef_stop_choices = [True,False]
    lagrange = 0.13
    bound = 0.14
    L_choices = [0.1,0.5,1]
    quadratic_choices = [True, False]
    shape = (20,)
    interactive = False

    def __init__(self, klass, mode):
        self.klass = klass
        self.mode = mode

    def __iter__(self):
        for offset, FISTA, coef_stop, L, q in itertools.product(self.offset_choices,
                                                                self.FISTA_choices,
                                                                self.coef_stop_choices,
                                                                self.L_choices,
                                                                self.quadratic_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            if self.mode == 'lagrange':
                atom = self.klass(self.shape, lagrange=self.lagrange)
            else:
                atom = self.klass(self.shape, bound=self.bound)

            if q: 
                atom.quadratic = rr.identity_quadratic(0,0,np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = Solver(atom,
                            interactive=self.interactive, 
                            coef_stop=coef_stop,
                            FISTA=FISTA,
                            L=L)

            # make sure certain lines of code are tested
            assert(atom == atom)
            atom.latexify(), atom.dual, atom.conjugate

            yield solver


class Solver(object):

    def __iter__(self):
        factory = SolverFactory(self.atom.__class__)
        for solver in factory:
            yield solver

    def __repr__(self):
        return 'Solver(%s, L=%f, prox_center=%s)' % (repr(self.atom), self.L, repr(self.prox_center))

    def __init__(self, atom, interactive=False, coef_stop=False,
                 FISTA=True, L=1, prox_center=None):
        self.atom = atom
        self.interactive = interactive
        self.coef_stop = coef_stop
        self.FISTA = FISTA
        self.L = L

        if prox_center is None:
            self.prox_center = np.random.standard_normal(atom.shape)
        else:
            self.prox_center = prox_center

        self.q = rr.identity_quadratic(L, self.prox_center, 0, 0)
        self.loss = rr.quadratic_loss.shift(self.prox_center, coef=L)

    @set_seed_for_test()
    def test_duality_of_projections(self):
        if self.atom.quadratic == rr.identity_quadratic(0,0,0,0) or self.atom.quadratic is None:

            tests = []

            d = self.atom.conjugate
            q = rr.identity_quadratic(1, self.prox_center, 0, 0)
            tests.append((self.prox_center-self.atom.proximal(q), d.proximal(q), 'testing duality of projections starting from atom\n %s ' % str(self)))

            if hasattr(self.atom, 'check_subgradient') and self.atom.offset is None:
                # check subgradient condition
                v1, v2 = self.atom.check_subgradient(self.atom, self.prox_center)
                tests.append((v1, v2, 'checking subgradient condition\n %s' % str(self)))

            if not self.interactive:
                for test in tests:
                    yield (all_close,) + test + (self,)
            else:
                for test in tests:
                    yield all_close(*((test + (self,))))

    @set_seed_for_test()
    def test_simple_problem_nonsmooth(self):
        tests = []
        atom, q = self.atom, self.q
        loss = self.loss

        p2 = copy(atom)
        p2.quadratic = atom.quadratic + q
        problem = rr.simple_problem.nonsmooth(p2)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-14, FISTA=self.FISTA, coef_stop=self.coef_stop, min_its=100)

        gg = rr.gengrad(problem, 2.) # this lipschitz constant is based on knowing our loss...
        tests.append((atom.proximal(q), gg, 'solving prox with gengrad\n %s ' % str(self)))

        tests.append((atom.proximal(q), atom.solve(q), 'solving prox with solve method\n %s ' % str(self)))

        tests.append((atom.proximal(q), solver.composite.coefs, 'solving prox with simple_problem.nonsmooth with monotonicity\n %s ' % str(self)))

        # use the solve method

        p3 = copy(atom)
        p3.quadratic = atom.quadratic + q
        soln = p3.solve(tol=1.e-14, min_its=10)
        tests.append((atom.proximal(q), soln, 'solving prox with solve method\n %s ' % str(self)))

        p4 = copy(atom)
        p4.quadratic = atom.quadratic + q
        problem = rr.simple_problem.nonsmooth(p4)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-14, monotonicity_restart=False, coef_stop=self.coef_stop,
                   FISTA=self.FISTA,
                   min_its=100)

        tests.append((atom.proximal(q), solver.composite.coefs, 'solving prox with simple_problem.nonsmooth with no monotonocity\n %s ' % str(self)))

        if not self.interactive:
            for test in tests:
                yield (all_close,) + test + (self,)
        else:
            for test in tests:
                yield all_close(*((test + (self,))))

    @set_seed_for_test()
    def test_simple_problem(self):
        tests = []
        atom, q, prox_center, L = self.atom, self.q, self.prox_center, self.L
        loss = self.loss

        problem = rr.simple_problem(loss, atom)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-12, FISTA=self.FISTA, coef_stop=self.coef_stop, min_its=100)

        tests.append((atom.proximal(q), solver.composite.coefs, 'solving prox with simple_problem with monotonicity\n %s' % str(self)))

        # write the loss in terms of a quadratic for the smooth loss and a smooth function...

        q = rr.identity_quadratic(L, prox_center, 0, 0)
        lossq = rr.quadratic_loss.shift(prox_center.copy(), coef=0.6*L)
        lossq.quadratic = rr.identity_quadratic(0.4*L, prox_center.copy(), 0, 0)
        problem = rr.simple_problem(lossq, atom)

        tests.append((atom.proximal(q), 
              problem.solve(coef_stop=self.coef_stop, 
                            FISTA=self.FISTA, 
                            tol=1.0e-12), 
               'solving prox with simple_problem ' +
               'with monotonicity  but loss has identity_quadratic %s\n ' % str(self)))

        problem = rr.simple_problem(loss, atom)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-12, monotonicity_restart=False,
                   coef_stop=self.coef_stop, FISTA=self.FISTA, min_its=100)

        tests.append((atom.proximal(q), solver.composite.coefs, 'solving prox with simple_problem no monotonicity_restart\n %s' % str(self)))

        d = atom.conjugate
        problem = rr.simple_problem(loss, d)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-12, monotonicity_restart=False, 
                   coef_stop=self.coef_stop, FISTA=self.FISTA, min_its=100)
        tests.append((d.proximal(q), problem.solve(tol=1.e-12,
                                                FISTA=self.FISTA,
                                                coef_stop=self.coef_stop,
                                                monotonicity_restart=False), 
               'solving dual prox with simple_problem no monotonocity\n %s ' % str(self)))

        if not self.interactive:
            for test in tests:
                yield (all_close,) + test + (self,)
        else:
            for test in tests:
                yield all_close(*((test + (self,))))

    @set_seed_for_test()
    def test_dual_problem(self):
        tests = []
        atom, q, prox_center, L = self.atom, self.q, self.prox_center, self.L
        loss = self.loss

        dproblem = rr.dual_problem.fromprimal(loss, atom)
        dcoef = dproblem.solve(coef_stop=self.coef_stop, tol=1.0e-14)
        tests.append((atom.proximal(q), dcoef, 'solving prox with dual_problem.fromprimal with monotonicity \n %s ' % str(self)))

        dproblem2 = rr.dual_problem(loss.conjugate, 
                                    rr.identity(loss.shape),
                                    atom.conjugate)
        dcoef2 = dproblem2.solve(coef_stop=self.coef_stop, tol=1.e-14)
        tests.append((atom.proximal(q), dcoef2, 'solving prox with dual_problem with monotonicity %s \n' % str(self)))

        if not self.interactive:
            for test in tests:
                yield (all_close,) + test + (self,)
        else:
            for test in tests:
                yield all_close(*((test + (self,))))

    @set_seed_for_test()
    def test_separable(self):
        tests = []
        atom, q, prox_center, L = self.atom, self.q, self.prox_center, self.L
        loss = self.loss

        problem = rr.separable_problem.singleton(atom, loss)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-12, 
                   coef_stop=self.coef_stop, FISTA=self.FISTA, min_its=100)

        tests.append((atom.proximal(q), solver.composite.coefs, 'solving atom prox with separable_atom.singleton \n%s ' % str(self)))


        d = atom.conjugate
        problem = rr.separable_problem.singleton(d, loss)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-12, 
                   coef_stop=self.coef_stop, FISTA=self.FISTA, min_its=100)

        tests.append((d.proximal(q), solver.composite.coefs, 'solving dual atom prox with separable_atom.singleton \n%s ' % str(self)))

        if not self.interactive:
            for test in tests:
                yield (all_close,) + test + (self,)
        else:
            for test in tests:
                yield all_close(*((test + (self,))))

    @set_seed_for_test()
    def test_container(self):
        tests = []
        atom, q, prox_center, L = self.atom, self.q, self.prox_center, self.L
        loss = self.loss

        problem = rr.container(loss, atom)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-12, 
                   coef_stop=self.coef_stop, FISTA=self.FISTA, min_its=100)

        tests.append((atom.proximal(q), solver.composite.coefs, 'solving atom prox with container\n %s ' % str(self)))

        # write the loss in terms of a quadratic for the smooth loss and a smooth function...

        q = rr.identity_quadratic(L, prox_center, 0, 0)
        lossq = rr.quadratic_loss.shift(prox_center.copy(), coef=0.6*L)
        lossq.quadratic = rr.identity_quadratic(0.4*L, prox_center.copy(), 0, 0)
        problem = rr.container(lossq, atom)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-12, FISTA=self.FISTA, coef_stop=self.coef_stop)

        tests.append((atom.proximal(q), 
                      problem.solve(tol=1.e-12,FISTA=self.FISTA,coef_stop=self.coef_stop), 
                      'solving prox with container with monotonicity ' + 
                      'but loss has identity_quadratic\n %s ' % str(self)))

        d = atom.conjugate
        problem = rr.container(d, loss)
        solver = rr.FISTA(problem)
        solver.fit(tol=1.0e-12, 
                   coef_stop=self.coef_stop, FISTA=self.FISTA, min_its=100)
        tests.append((d.proximal(q), solver.composite.coefs, 'solving dual prox with container\n %s ' % str(self)))

        if not self.interactive:
            for test in tests:
                yield (all_close,) + test + (self,)
        else:
            for test in tests:
                yield all_close(*((test + (self,))))

    def all_tests(self):
        for group in [self.test_duality_of_projections,
                      self.test_simple_problem,
                      self.test_separable,
                      self.test_dual_problem,
                      self.test_container,
                      self.test_simple_problem_nonsmooth
                      ]:
            for t in group():
                yield t



import numpy as np
from scipy import sparse
from copy import copy
import warnings

from ..problems.composite import composite, nonsmooth
from .cones import cone, affine_cone
from ..identity_quadratic import identity_quadratic
from ..atoms import _work_out_conjugate
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import doc_template_user

@objective_doc_templater()
class linear_constraint(cone):

    """
    This class allows specifications of linear constraints
    of the form :math:`x \in \text{row}(L)` by specifying
    an orthonormal basis for the rowspace of :math:`L`.

    If the constraint is of the form :math:`Ax=0`, then
    this linear constraint can be created using the
    *linear* classmethod of the *zero* cone in *regreg.cones*.
    
    """
    tol = 1.0e-05

    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'projection'
    objective_vars['dualconeklass'] = 'projection_complement'
    objective_vars['initargs'] = '(4,), [[1,0,0,0],[0,1,0,0]]'

    #XXX should basis by a linear operator instead?
    def __init__(self, shape, basis,
                 offset=None,
                 initial=None, 
                 quadratic=None):

        cone.__init__(self,
                      shape,
                      offset=offset,
                      initial=initial,
                      quadratic=quadratic)
        self.basis = np.asarray(basis)
        

    def __eq__(self, other):
        if self.__class__ == other.__class__:
            return (self.shape == other.shape
                    and np.allclose(other.basis, self.basis))
        return False

    def __copy__(self):
        return self.__class__(copy(self.shape),
                              self.basis.copy(),
                              offset=copy(self.offset),
                              quadratic=self.quadratic)
    
    def __repr__(self):
        if self.quadratic.iszero:
            return "%s(%s, %s, offset=%s)" % \
                (self.__class__.__name__,
                 repr(self.basis),
                 repr(self.shape), 
                 repr(self.offset))
        else:
            return "%s(%s, %s, offset=%s, quadratic=%s)" % \
                (self.__class__.__name__,
                 repr(self.basis),
                 repr(self.shape), 
                 repr(self.offset),
                 self.quadratic)

    @doc_template_user
    def get_conjugate(self):
        if self.quadratic.coef == 0:

            offset, outq = _work_out_conjugate(self.offset, self.quadratic)

            cls = conjugate_cone_pairs[self.__class__]
            atom = cls(self.shape, 
                       self.basis,
                       offset=offset,
                       quadratic=outq)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    @classmethod
    @doc_template_user
    def linear(cls, linear_operator, basis, diag=False,
               linear_term=None, offset=None):
        l = linear_transform(linear_operator, diag=diag)
        cone = cls(l.input_shape, basis,
                   linear_term=linear_term, offset=offset)
        return affine_cone(cone, l)

@objective_doc_templater()
class projection(linear_constraint):

    objective_vars = linear_constraint.objective_vars.copy()
    objective_vars['coneklass'] = 'projection'
    objective_vars['dualconeklass'] = 'projection_complement'

    """
    An atom representing a linear constraint.
    It is specified via a matrix that is assumed
    to be an set of row vectors spanning the space.

    Notes
    =====

    It is assumed (without checking) that the rows of basis
    are orthonormal, so that projecting *x* onto their span
    is simply np.dot(basis.T, np.dot(basis, x))
    """

    @doc_template_user
    def constraint(self, x):
        projx = self.cone_prox(x)
        incone = np.linalg.norm(x-projx) / max([np.linalg.norm(x),1]) < self.tol
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, x,  lipschitz=1):
        coefs = np.dot(self.basis, x)
        return np.dot(coefs, self.basis)

    @doc_template_user
    def proximal(self, proxq, prox_control=None):
        return cone.proximal(self, proxq, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class projection_complement(linear_constraint):

    """
    An atom representing a linear constraint.
    The orthogonal complement of projection, it is specified
    with an orthonormal basis for the complement
    """

    objective_vars = linear_constraint.objective_vars.copy()
    objective_vars['coneklass'] = 'projection_complement'
    objective_vars['dualconeklass'] = 'projection'

    @doc_template_user
    def constraint(self, x):
        projx = self.cone_prox(x)
        incone = np.linalg.norm(projx) / max([np.linalg.norm(x),1]) < self.tol
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, x,  lipschitz=1):
        coefs = np.dot(self.basis, x)
        return x - np.dot(coefs, self.basis)

    @doc_template_user
    def proximal(self, proxq, prox_control=None):
        return cone.proximal(self, proxq, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

conjugate_cone_pairs = {}
for n1, n2 in [(projection, projection_complement),
               ]:
    conjugate_cone_pairs[n1] = n2
    conjugate_cone_pairs[n2] = n1
